/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.ImageIcon;
import javax.wsdl.Binding;
import javax.wsdl.BindingOperation;
import javax.wsdl.BindingOutput;
import javax.wsdl.Definition;
import javax.wsdl.extensions.mime.MIMEContent;
import javax.wsdl.extensions.mime.MIMEMultipartRelated;
import javax.wsdl.extensions.mime.MIMEPart;
import javax.xml.namespace.QName;

import org.apache.xmlbeans.SchemaType;

import com.eviware.soapui.config.CallConfig;
import com.eviware.soapui.config.OperationConfig;
import com.eviware.soapui.config.PartsConfig.Part;
import com.eviware.soapui.impl.wsdl.actions.operation.AddToMockServiceAction;
import com.eviware.soapui.impl.wsdl.actions.operation.NewRequestAction;
import com.eviware.soapui.impl.wsdl.actions.operation.RelabelOperationAction;
import com.eviware.soapui.impl.wsdl.actions.support.ShowOnlineHelpAction;
import com.eviware.soapui.impl.wsdl.support.HelpUrls;
import com.eviware.soapui.impl.wsdl.support.soap.SoapMessageBuilder;
import com.eviware.soapui.impl.wsdl.support.wsdl.WsdlContext;
import com.eviware.soapui.impl.wsdl.support.wsdl.WsdlUtils;
import com.eviware.soapui.model.iface.MessagePart;
import com.eviware.soapui.model.iface.Operation;
import com.eviware.soapui.model.iface.Request;
import com.eviware.soapui.model.iface.MessagePart.FaultPart;
import com.eviware.soapui.support.UISupport;
import com.eviware.soapui.support.action.ActionSupport;

/**
 * WSDL implementation of Operation, maps to a WSDL BindingOperation
 * 
 * @author Ole.Matzura
 */

public class WsdlOperation extends AbstractWsdlModelItem<OperationConfig> implements Operation
{
   public static final String STYLE_DOCUMENT = "Document";
	public static final String STYLE_RPC = "RPC";
	private List<WsdlRequest> requests = new ArrayList<WsdlRequest>();
   private WsdlInterface iface;
	private ImageIcon oneWayIcon;

   public WsdlOperation( WsdlInterface iface, OperationConfig operationConfig )
   {
   	super( operationConfig, iface, "/operation.gif" );
      this.iface = iface;
      
      if( operationConfig.isSetIsOneWay())
      {
      	operationConfig.setIsOneWay( false );
      }
      
      List<CallConfig> requestConfigs = getConfig().getCallList();
      for (CallConfig config : requestConfigs)
      {
         requests.add( new WsdlRequest( this, config));
      }

      addAction( new AddToMockServiceAction( this ));
//      addAction( new AddAsMockResponseStepAction(this));
      addAction( ActionSupport.SEPARATOR_ACTION );
      addAction( new NewRequestAction( this ));
      addAction( new RelabelOperationAction( this ));
      addAction( ActionSupport.SEPARATOR_ACTION );
      addAction( new ShowOnlineHelpAction( HelpUrls.OPERATION_HELP_URL ));
      
      oneWayIcon = UISupport.createImageIcon( "/onewayoperation.gif" );
   }
   
	public String getAction()
   {
      return getConfig().getAction();
   }

   public WsdlRequest getRequestAt(int index)
   {
      return requests.get( index );
   }
   
   public WsdlRequest getRequestByName(String requestName)
	{
		return (WsdlRequest) getWsdlModelItemByName( requests, requestName );
	}

	public int getRequestCount()
   {
      return requests.size();
   }

   public ImageIcon getIcon()
	{
		if( isOneWay() )
			return oneWayIcon;
		else
			return super.getIcon();
	}

	public WsdlRequest addNewRequest( String name )
   {
      WsdlRequest requestImpl = new WsdlRequest( this, getConfig().addNewCall() );
      requestImpl.setName( name );
      requests.add( requestImpl );
      ((WsdlInterface)getInterface()).fireRequestAdded( requestImpl );
      return requestImpl;
   }

   public WsdlInterface getInterface()
   {
      return iface;
   }

   public void setAction(String soapAction)
   {
      String old = getAction();
      getConfig().setAction( soapAction );
      notifyPropertyChanged( ACTION_PROPERTY, old, soapAction );
   }

   public String createRequest( boolean buildOptional )
   {
      if( iface.getBindingName() == null )
      {
      	UISupport.showErrorMessage( "Missing binding name, please try to refresh " +
               "Interface\nfor request generation to work correctly" );
         return null;
      }
      
      if( getBindingOperationName() == null )
      {
      	UISupport.showErrorMessage(  "Missing bindingOperation name, please try to refresh " +
               "Interface\nfor request generation to work correctly" );
         return null;
      }
      
      try
      {
         SoapMessageBuilder builder = (SoapMessageBuilder) iface.getMessageBuilder();
         BindingOperation bindingOperation = findBindingOperation( iface.getWsdlContext().getDefinition() );
         
         if( bindingOperation == null )
         {
         	UISupport.showErrorMessage( "Failed to find bindingOperation, please try to refresh " +
                  "Interface\nfor request generation to work correctly" );
            return null;
         }
         
			return builder.buildSoapRequest( bindingOperation, buildOptional );
      }
      catch (Exception e)
      {
         e.printStackTrace();
         return null;
      }
   }
   
   public String createResponse( boolean buildOptional )
   {
      if( iface.getBindingName() == null )
      {
      	UISupport.showErrorMessage( "Missing binding name, please try to refresh " +
               "Interface\nfor request generation to work correctly" );
         return null;
      }
      
      if( getBindingOperationName() == null )
      {
      	UISupport.showErrorMessage(  "Missing bindingOperation name, please try to refresh " +
               "Interface\nfor request generation to work correctly" );
         return null;
      }
      
      try
      {
         SoapMessageBuilder builder = (SoapMessageBuilder) iface.getMessageBuilder();
         BindingOperation bindingOperation = findBindingOperation( iface.getWsdlContext().getDefinition() );
         
         if( bindingOperation == null )
         {
         	UISupport.showErrorMessage( "Failed to find bindingOperation, please try to refresh " +
                  "Interface\nfor request generation to work correctly" );
            return null;
         }
         
			return builder.buildSoapResponse( bindingOperation, buildOptional );
      }
      catch (Exception e)
      {
         e.printStackTrace();
         return null;
      }
   }
   
   public BindingOperation findBindingOperation(Definition definition)
   {
      Binding binding = definition.getBinding( iface.getBindingName() );
      if( binding == null )
      	return null;
      
		String inputName = getInputName();
		String outputName = getOutputName();
		
		if( inputName == null )
			inputName = ":none";
		
		if( outputName == null )
			outputName = ":none";
		
		BindingOperation result = binding.getBindingOperation( getConfig().getBindingOperationName(), inputName, outputName );

		if( result == null && (inputName.equals( ":none" ) || outputName.equals( ":none" )))
		{
			// fall back to this behaviour for WSDL4j 1.5.0 compatibility
			result = binding.getBindingOperation( getConfig().getBindingOperationName(), 
						inputName.equals( ":none" ) ? null : inputName, outputName.equals( ":none" ) ? null : outputName ); 
		}
		return result;
   }
  
   public void removeRequest( WsdlRequest request )
   {
      int ix = requests.indexOf( request );
      requests.remove( ix );

      try
      {
      	((WsdlInterface)getInterface()).fireRequestRemoved( request );
      }
      finally
      {
	      request.release();
	      getConfig().removeCall( ix );
      }
   }

   public String getBindingOperationName()
   {
      return getConfig().getBindingOperationName();
   }
   
   public void setBindingOperationName( String name )
   {
      getConfig().setBindingOperationName( name );
   }
   
   public void setInputName( String name )
   {
   	getConfig().setInputName( name );
   }

   public String getInputName()
   {
   	String inputName = getConfig().getInputName();
   	return inputName == null || inputName.trim().length() == 0 ? null : inputName;
   }
   
   public void setOutputName( String name )
   {
   	if( name == null )
   	{
   		if( getConfig().isSetOutputName() )
   			getConfig().unsetOutputName();
   	}
   	else
   		getConfig().setOutputName( name );
   }

   public String getOutputName()
   {
   	String outputName = getConfig().getOutputName();
   	return outputName == null || outputName.trim().length() == 0 ? null : outputName;
   }
   
   public void setOneWay( boolean isOneWay )
   {
   	getConfig().setIsOneWay( isOneWay );
   }
   
   public boolean isOneWay()
   {
   	return getConfig().getIsOneWay();
   }
   
	public void initFromBindingOperation(BindingOperation operation, boolean notifyUpdate )
	{
      setAction( WsdlUtils.getSoapAction( operation ));
      setName( operation.getOperation().getName() );
      setBindingOperationName( operation.getName() );
      setInputName( operation.getBindingInput().getName() );
      
      BindingOutput bindingOutput = operation.getBindingOutput();
      
      // bindingOutput is null for oneway operations
		if( bindingOutput != null )
      	setOutputName( bindingOutput.getName() );
		else
			setOutputName( null );

      setOneWay( bindingOutput == null );
      
      initAttachments(operation);
      
      if( notifyUpdate )
      {
      	iface.fireOperationUpdated( this );
      }
	}

	private void initAttachments(BindingOperation operation)
	{
		if( getConfig().isSetRequestParts() )
			getConfig().unsetRequestParts();
		
		if( getConfig().isSetResponseParts() )
			getConfig().unsetResponseParts();
		
		BindingOutput bindingOutput = operation.getBindingOutput();
		
		if( bindingOutput != null )
	   {
		   MIMEMultipartRelated multipartOutput = (MIMEMultipartRelated) WsdlUtils.getExtensiblityElement( 
		    		bindingOutput.getExtensibilityElements(), MIMEMultipartRelated.class );
		 
		   getConfig().setReceivesAttachments( multipartOutput != null );
	   }
		
		MIMEMultipartRelated multipartInput = (MIMEMultipartRelated) WsdlUtils.getExtensiblityElement( 
      		operation.getBindingInput().getExtensibilityElements(), MIMEMultipartRelated.class );
 
      getConfig().setSendsAttachments( multipartInput != null );
      if( multipartInput != null )
      {
      	List<MIMEPart> parts = multipartInput.getMIMEParts();
      	Map<String,Part> partMap = new HashMap<String,Part>();

      	for( int c = 0; c < parts.size(); c++ )
      	{
	      	List<MIMEContent> contentParts = WsdlUtils.getExtensiblityElements( parts.get(c).getExtensibilityElements(), MIMEContent.class );
	      	
	      	for( MIMEContent content : contentParts )
	      	{
	      		Part part = partMap.get( content.getPart());
	      		if( part != null )
	      		{ 
	      			if( !part.getContentTypeList().contains( content.getType() ))
	      				part.addContentType( content.getType() );
	      		}
	      		else
	      		{
	      			if( !getConfig().isSetRequestParts() )
	      				getConfig().addNewRequestParts();
	      			
	      			Part requestPart = getConfig().getRequestParts().addNewPart();
	      			requestPart.addContentType( content.getType() );
	      			requestPart.setName( content.getPart() );
	      			
	      			partMap.put( requestPart.getName(), requestPart );
	      		}
	      	}
      	}
      }
	}
	
	public boolean getReceivesAttachments()
	{
		return getConfig().getReceivesAttachments();
	}
	
	public boolean getSendsAttachments()
	{
		return getConfig().getSendsAttachments();
	}
	
	public QName getRequestBodyElementQName() throws Exception
	{
		WsdlInterface iface = (WsdlInterface) getInterface();
		
		Definition definition = iface.getWsdlContext().getDefinition();
		BindingOperation bindingOperation = findBindingOperation( definition );
		if( WsdlUtils.isRpc( definition, bindingOperation))
		{
			String ns = WsdlUtils.getSoapBodyNamespace( bindingOperation.getBindingInput().getExtensibilityElements() );
	      if( ns == null )
	      {
	      	ns = definition.getTargetNamespace();
	      }
			
			return new QName( ns, bindingOperation.getName() );
		}
		else
		{
			List<javax.wsdl.Part> parts = bindingOperation.getOperation().getInput().getMessage().getOrderedParts( null );
			if( parts == null || parts.isEmpty() )
				return null;
			
			javax.wsdl.Part part = parts.get( 0 );
			
			if( part.getElementName() != null )
			{
				return part.getElementName();
			}
			else
			{
				return new QName( definition.getTargetNamespace(), part.getName() );	
			}
		}
	}
	
	public QName getResponseBodyElementQName() throws Exception
	{
		if( isOneWay() )
			return null;
		
		WsdlInterface iface = (WsdlInterface) getInterface();
		
		Definition definition = iface.getWsdlContext().getDefinition();
		BindingOperation bindingOperation = findBindingOperation( definition );
		if( WsdlUtils.isRpc( definition, bindingOperation))
		{
			String ns = WsdlUtils.getSoapBodyNamespace( bindingOperation.getBindingOutput().getExtensibilityElements() );
	      if( ns == null )
	      {
	      	ns = definition.getTargetNamespace();
	      }
			
			return new QName( ns, bindingOperation.getName() + "Response" );
		}
		else
		{
			List<javax.wsdl.Part> parts = bindingOperation.getOperation().getOutput().getMessage().getOrderedParts( null );
			if( parts == null || parts.isEmpty() )
				return null;
			
			javax.wsdl.Part part = parts.get( 0 );
			
			if( part.getElementName() != null )
			{
				return part.getElementName();
			}
			else
			{
				return new QName( definition.getTargetNamespace(), part.getName() );	
			}
		}
	}
	
	public String getStyle()
	{
		WsdlContext wsdlContext = iface.getWsdlContext();
		if( !wsdlContext.isLoaded() )
			return "<not loaded>";
		
		try
		{
			Definition definition = wsdlContext.getDefinition();
			BindingOperation bindingOperation = findBindingOperation( definition);
			
			if( bindingOperation == null )
				return "<missing bindingOperation>";
			
			if( WsdlUtils.isRpc( definition, bindingOperation ))
			{
				return WsdlOperation.STYLE_RPC;
			}
			else
			{
				return WsdlOperation.STYLE_DOCUMENT;
			}
		}
		catch (Exception e)
		{
			e.printStackTrace();
			return "<error>";
		}
	}

	public void release()
	{
		super.release();
		
		for( WsdlRequest request : requests )
			request.release();
	}

	public BindingOperation getBindingOperation()
	{
		try
		{
			return findBindingOperation( ((WsdlInterface)getInterface()).getWsdlContext().getDefinition() );
		}
		catch( Exception e )
		{
			e.printStackTrace();
			return null;
		}
	}

	public List<Request> getRequests()
	{
		return new ArrayList<Request>( requests );
	}

	public MessagePart [] getFaultParts()
	{
		BindingOperation bindingOperation = getBindingOperation();
		Map bindingFaults = bindingOperation.getBindingFaults();

		List<MessagePart> result = new ArrayList<MessagePart>();
		for( Object key : bindingFaults.keySet() )
		{
			result.add( new WsdlFaultPart( (String)key ));
		}
	
		return result.toArray( new MessagePart[result.size()] );
	}
	
	private class WsdlFaultPart extends FaultPart
	{
		private final String name;

		public WsdlFaultPart( String name )
		{
			this.name = name;
		}

		@Override
		public javax.wsdl.Part[] getWsdlParts()
		{
			return WsdlUtils.getFaultParts( getBindingOperation(), name );
		}

		@Override
		public QName getPartElement()
		{
			return null;
		}

		public String getDescription()
		{
			return null;
		}

		public String getName()
		{
			return name;
		}

		@Override
		public SchemaType getSchemaType()
		{
			return null;
		}}
}
