/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.swing.ImageIcon;
import javax.xml.namespace.QName;

import org.apache.log4j.Logger;
import org.apache.xmlbeans.XmlException;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.config.InterfaceConfig;
import com.eviware.soapui.config.MockServiceConfig;
import com.eviware.soapui.config.ProjectConfig;
import com.eviware.soapui.config.SoapuiProjectDocumentConfig;
import com.eviware.soapui.config.TestSuiteConfig;
import com.eviware.soapui.impl.WorkspaceImpl;
import com.eviware.soapui.impl.wsdl.actions.iface.tools.jbossws.WSToolsJava2WsdlAction;
import com.eviware.soapui.impl.wsdl.actions.project.AddInterfaceActionFromFile;
import com.eviware.soapui.impl.wsdl.actions.project.AddInterfaceActionFromURL;
import com.eviware.soapui.impl.wsdl.actions.project.AddNewMockServiceAction;
import com.eviware.soapui.impl.wsdl.actions.project.AddNewTestSuiteAction;
import com.eviware.soapui.impl.wsdl.actions.project.ReloadProjectAction;
import com.eviware.soapui.impl.wsdl.actions.project.RemoveProjectAction;
import com.eviware.soapui.impl.wsdl.actions.project.RenameProjectAction;
import com.eviware.soapui.impl.wsdl.actions.project.SaveProjectAction;
import com.eviware.soapui.impl.wsdl.actions.project.SaveProjectAsAction;
import com.eviware.soapui.impl.wsdl.actions.support.ShowOnlineHelpAction;
import com.eviware.soapui.impl.wsdl.mock.WsdlMockService;
import com.eviware.soapui.impl.wsdl.support.HelpUrls;
import com.eviware.soapui.impl.wsdl.support.wsdl.WsdlImporter;
import com.eviware.soapui.model.iface.Interface;
import com.eviware.soapui.model.mock.MockService;
import com.eviware.soapui.model.project.Project;
import com.eviware.soapui.model.project.ProjectListener;
import com.eviware.soapui.model.settings.Settings;
import com.eviware.soapui.model.testsuite.TestSuite;
import com.eviware.soapui.settings.ProjectSettings;
import com.eviware.soapui.settings.UISettings;
import com.eviware.soapui.settings.WsdlSettings;
import com.eviware.soapui.support.SoapUIException;
import com.eviware.soapui.support.Tools;
import com.eviware.soapui.support.UISupport;
import com.eviware.soapui.support.action.ActionSupport;

/**
 * WSDL project implementation
 * 
 * @author Ole.Matzura
 */

public class WsdlProject extends AbstractWsdlModelItem<ProjectConfig> implements Project
{
   private WorkspaceImpl workspace;
   private String path;
   private List<WsdlInterface> interfaces = new ArrayList<WsdlInterface>();
   private List<WsdlTestSuite> testSuites = new ArrayList<WsdlTestSuite>();
   private List<WsdlMockService> mockServices = new ArrayList<WsdlMockService>();
   private Set<ProjectListener> listeners = new HashSet<ProjectListener>();
   private SoapuiProjectDocumentConfig projectDocument;
	private ImageIcon disabledIcon;
	private long lastModified;
   
   private final static Logger log = Logger.getLogger( WsdlProject.class );

   public WsdlProject() throws XmlException, IOException
   {
   	this( (WorkspaceImpl)null );
   }

   public WsdlProject( String path ) throws XmlException, IOException, SoapUIException
   {
   	this( path, null );
   }
   
   public WsdlProject( WorkspaceImpl workspace ) throws XmlException, IOException 
   {
   	super( null, workspace, "/project.gif" );
      this.workspace = workspace;
   	projectDocument = SoapuiProjectDocumentConfig.Factory.newInstance();
   	
   	setConfig( projectDocument.addNewSoapuiProject() );
      
      initActions();
   }

   public WsdlProject(String path, WorkspaceImpl workspace ) throws SoapUIException
   {
   	this( path, workspace, true );
   }
   
   public WsdlProject(String path, WorkspaceImpl workspace, boolean create ) throws SoapUIException
   {
   	super( null, workspace, "/project.gif" );
   	
      this.workspace = workspace;
   	this.path = path;
      File file = new File(path);
      
      if( file.exists())
      {
         loadProject( file );
      }
      else if( create )
      {
         projectDocument = SoapuiProjectDocumentConfig.Factory.newInstance();
         setConfig( projectDocument.addNewSoapuiProject() );
      }
      else
      {
      	disabledIcon = UISupport.createImageIcon( "/disabledProject.gif" );
      }
      
      setProjectRoot(path);
     	initActions();
   }

	private void loadProject( File file ) throws SoapUIException
	{
		try
		{
			lastModified = file.lastModified();
			projectDocument = SoapuiProjectDocumentConfig.Factory.parse( file );
		}
		catch (Exception e)
		{
			throw new SoapUIException( "Failed to create project from file [" + file.getName() + "]", e );
		}
		
		setConfig( projectDocument.getSoapuiProject() );
		
		// removed cached definitions if caching is disabled
		if( !getSettings().getBoolean( WsdlSettings.CACHE_WSDLS ))
		{
			removeDefinitionCaches( projectDocument );
		}
		
		log.info( "Loaded project from [" + file.getAbsolutePath() + "]" );
		 
		List<InterfaceConfig> interfaceConfigs = getConfig().getInterfaceList();
		for (InterfaceConfig config : interfaceConfigs)
		{
		   interfaces.add( new WsdlInterface( this, config ));
		}

		List<TestSuiteConfig> testSuiteConfigs = getConfig().getTestSuiteList();
		for( TestSuiteConfig config : testSuiteConfigs )
		{
		   testSuites.add( new WsdlTestSuite( this, config ));
		}
		
		List<MockServiceConfig> mockServiceConfigs = getConfig().getMockServiceList();
		for( MockServiceConfig config : mockServiceConfigs )
		{
		   mockServices.add( new WsdlMockService( this, config ));
		}
	}

	private void setProjectRoot(String path)
	{
		if( path != null && projectDocument != null )
      {
      	int ix = path.lastIndexOf( File.separatorChar );
      	if( ix > 0 )
      		getSettings().setString( ProjectSettings.PROJECT_ROOT, path.substring( 0, ix ) );
      }
	}
	
	@Override
	public ImageIcon getIcon()
	{
		if( projectDocument != null )
			return super.getIcon();
		else
			return disabledIcon;
	}
	
	@Override
	public String getName()
	{
		if( projectDocument != null )
			return super.getName();
		else
		{
			int ix = path.lastIndexOf( File.separatorChar );
			String name = ix == -1 ? path : path.substring( ix+1 );
			return name + " (disabled)";
		}
	}
	
	@Override
	public String getDescription()
	{
		return projectDocument == null ? getName() : super.getDescription();
	}

	private void initActions()
	{
		if( projectDocument == null )
		{
			addAction( new ReloadProjectAction( this ));
			addAction( new RemoveProjectAction( this ));
			addAction( ActionSupport.SEPARATOR_ACTION );
	      addAction( new ShowOnlineHelpAction( HelpUrls.PROJECT_HELP_URL ));
		}
		else
		{
	      addAction( new AddInterfaceActionFromURL( this ) );
	      addAction( new AddInterfaceActionFromFile( this ) );
	      addAction( ActionSupport.SEPARATOR_ACTION );
	      addAction( new WSToolsJava2WsdlAction( this ) );
	      addAction( ActionSupport.SEPARATOR_ACTION );
	      addAction( new AddNewTestSuiteAction( this ) );
	      addAction( new AddNewMockServiceAction( this ) );
	      addAction( ActionSupport.SEPARATOR_ACTION );
	      addAction( new RenameProjectAction( this ) );
	      addAction( new RemoveProjectAction( this ) );
	      addAction( new ReloadProjectAction( this ));
	      addAction( ActionSupport.SEPARATOR_ACTION );
	      addAction( new SaveProjectAction( this ));
	      addAction( new SaveProjectAsAction( this ));
	      addAction( ActionSupport.SEPARATOR_ACTION );
	      addAction( new ShowOnlineHelpAction( HelpUrls.PROJECT_HELP_URL ));
		}
	}
	
   public WorkspaceImpl getWorkspace()
   {
      return workspace;
   }

	public WsdlInterface getInterfaceAt(int index)
   {
      return interfaces.get( index );
   }

   public WsdlInterface getInterfaceByName(String interfaceName)
	{
		return (WsdlInterface) getWsdlModelItemByName( interfaces, interfaceName );
	}

   public WsdlInterface getInterfaceByBindingName( QName bindingName )
   {
   	for( int c = 0; c < getInterfaceCount(); c++ )
   	{
   		if( getInterfaceAt( c ).getBindingName().equals( bindingName ))
   			return getInterfaceAt( c );
   	}
   	
   	return null;
   }
   
	public int getInterfaceCount()
   {
      return interfaces.size();
   }

   public String getPath()
	{
		return path;
	}

	public boolean save() throws IOException 
   {
		if( projectDocument == null )
			return true;
		
      if( path == null )
      {
         File file = UISupport.getFileDialogs().saveAs(this, "Save project " + getName());
         if( file == null ) return false;

         path = file.getAbsolutePath();
      }
      
      File projectFile = new File( path );
      
      while( projectFile.exists() && !projectFile.canWrite() )
      {
      	if( UISupport.confirm( "Project file [" + path + "] can not be written to, save to new file?", "Save Project" ))
      	{
      		projectFile = UISupport.getFileDialogs().saveAs(this, "Save project " + getName());
            if( projectFile == null ) return false;

            path = projectFile.getAbsolutePath();
      	}
      	else return false;
      }
      
      
      // check modified 
		if( projectFile.exists() && lastModified != 0 && lastModified < projectFile.lastModified() )
		{
			if( !UISupport.confirm( 
						"Project file for [" + getName() + "] has been modified externally, overwrite?", 
						"Save Project" ))
				return false;
		}
      
   	long size = 0;
   	
   	if( projectFile.exists() && getSettings().getBoolean( UISettings.CREATE_BACKUP ))
   	{
   		createBackup( projectFile );
   	}
   	
   	// check for caching
   	if( !getSettings().getBoolean( WsdlSettings.CACHE_WSDLS ))
   	{
   		// no caching -> create copy and remove definition cachings
   		SoapuiProjectDocumentConfig config = (SoapuiProjectDocumentConfig) projectDocument.copy();
   		removeDefinitionCaches(config);
   		
   		config.getSoapuiProject().setSoapuiVersion( SoapUI.SOAPUI_VERSION );
   		config.save( projectFile );
   	}
   	else
   	{
      	try
			{
      		// save to temporary buffer to avoid corruption of file
				projectDocument.getSoapuiProject().setSoapuiVersion( SoapUI.SOAPUI_VERSION );
				ByteArrayOutputStream writer = new ByteArrayOutputStream( 8192 );
				projectDocument.save( writer );
				FileOutputStream out = new FileOutputStream( projectFile );
				writer.writeTo( out );
				out.close();
				size = writer.size();
			}
			catch (Throwable t)
			{
				t.printStackTrace();
				UISupport.showErrorMessage( "Failed to save project [" + getName() + "]: " + t.toString() );
				return false;
			}
   	}
   	
   	lastModified = projectFile.lastModified();
		log.info( "Saved project [" + getName() + "] to [" + projectFile.getAbsolutePath() + " - " +
				size + " bytes" );
		setProjectRoot(path);
		return true;
}

	private void createBackup(File projectFile) throws IOException
	{
		String backupFolderName = getSettings().getString( UISettings.BACKUP_FOLDER, "" );
		
		File backupFolder = new File( backupFolderName );
		if( !backupFolder.isAbsolute() )
		{
			backupFolder = new File( projectFile.getParentFile(), backupFolderName );
		}
		
		if( !backupFolder.exists() )
			backupFolder.mkdirs();
		
		File backupFile = new File( backupFolder, projectFile.getName() + ".backup" );
		log.info( "Backing up [" + projectFile + "] to [" + backupFile + "]" );
		
		Tools.copyFile( projectFile, backupFile, true );
	}

	private void removeDefinitionCaches(SoapuiProjectDocumentConfig config)
	{
		for( InterfaceConfig ifaceConfig : config.getSoapuiProject().getInterfaceList() )
		{
			if( ifaceConfig.isSetDefinitionCache() )
			{
				log.info( "Removing definition cache from interface [" + ifaceConfig.getName() + "]" );
				ifaceConfig.unsetDefinitionCache();
			}
		}
	}
	
	public WsdlInterface [] importWsdl( String url, boolean createRequests ) throws SoapUIException
	{
		return importWsdl( url, createRequests, null );
	}
   
   public WsdlInterface [] importWsdl( String url, boolean createRequests, QName bindingName ) throws SoapUIException
   {
   	if( projectDocument == null )
   		return null;
   	
   	WsdlInterface[] result;
   	
      try
      {
         result = WsdlImporter.getInstance().importWsdl( this, url, bindingName );
      }
      catch (Exception e)
      {
         log.error( "Error importing wsdl: " + e );
         e.printStackTrace();
         throw new SoapUIException( "Error importing wsdl", e );
      }
      
      try
      {      
			if( createRequests && result != null )
         {
         	for (WsdlInterface iface : result)
				{
         		for( int c = 0; c < iface.getOperationCount(); c++ )
         		{
         			WsdlOperation operation = (WsdlOperation) iface.getOperationAt( c );
         			WsdlRequest request = operation.addNewRequest( "Request 1");
                  try
                  {
                     request.setRequestContent( operation.createRequest( true ));
                  }
                  catch (Exception e)
                  {
                     e.printStackTrace();
                  }
         		}
				}
         }
      }
      catch (Exception e)
      {
      	log.error( "Error creating requests: " + e.getMessage() );
         throw new SoapUIException( "Error creating requests", e );
      }
      
      return result;
   }
   
   public WsdlInterface addNewInterface( String name )
   {
      WsdlInterface iface = new WsdlInterface( this, getConfig().addNewInterface());
      iface.setName( name );
      interfaces.add( iface );
      fireInterfaceAdded( iface );

      return iface;
   }
   
   public void addProjectListener(ProjectListener listener)
   {
      listeners.add( listener );
   }
   
   public void removeProjectListener(ProjectListener listener)
   {
      listeners.remove( listener );
   }
   
   public void fireInterfaceAdded( WsdlInterface iface )
   {
      ProjectListener[] a = listeners.toArray( new ProjectListener[listeners.size()] );
      
      for (int c = 0; c < a.length; c++ )
      {
         a[c].interfaceAdded( iface );
      }
   }
   
   public void fireInterfaceRemoved( WsdlInterface iface )
   {
      ProjectListener[] a = listeners.toArray( new ProjectListener[listeners.size()] );
      
      for (int c = 0; c < a.length; c++ )
      {
         a[c].interfaceRemoved( iface );
      }
   }
   
   
   public void fireTestSuiteAdded( WsdlTestSuite testSuite )
   {
      ProjectListener[] a = listeners.toArray( new ProjectListener[listeners.size()] );
      
      for (int c = 0; c < a.length; c++ )
      {
         a[c].testSuiteAdded( testSuite );
      }
   }
   
   public void fireTestSuiteRemoved( WsdlTestSuite testSuite )
   {
      ProjectListener[] a = listeners.toArray( new ProjectListener[listeners.size()] );
      
      for (int c = 0; c < a.length; c++ )
      {
         a[c].testSuiteRemoved( testSuite );
      }
   } 
   
   public void fireMockServiceAdded( WsdlMockService mockService )
   {
      ProjectListener[] a = listeners.toArray( new ProjectListener[listeners.size()] );
      
      for (int c = 0; c < a.length; c++ )
      {
         a[c].mockServiceAdded( mockService );
      }
   }
   
   public void fireMockServiceRemoved( WsdlMockService mockService )
   {
      ProjectListener[] a = listeners.toArray( new ProjectListener[listeners.size()] );
      
      for (int c = 0; c < a.length; c++ )
      {
         a[c].mockServiceRemoved( mockService );
      }
   }
   
   public void removeInterface(WsdlInterface iface )
   {
      int ix = interfaces.indexOf( iface );
      interfaces.remove( ix );
      try
      {
      	fireInterfaceRemoved( iface );
      }
      finally
      {
      	iface.release();
      	getConfig().removeInterface( ix );
      }
   }
   
   public void removeTestSuite(WsdlTestSuite testSuite )
   {
      int ix = testSuites.indexOf( testSuite );
      testSuites.remove( ix );
      
      try
      {
      	fireTestSuiteRemoved( testSuite );
      }
      finally
      {
      	testSuite.release();
      	getConfig().removeTestSuite( ix );
      }
   }

   public boolean isDisabled()
   {
   	return projectDocument == null;
   }
   
   public int getTestSuiteCount()
   {
      return testSuites.size();
   }

   public WsdlTestSuite getTestSuiteAt(int index)
   {
      return testSuites.get( index );
   }

   public WsdlTestSuite getTestSuiteByName(String testSuiteName)
	{
		return  ( WsdlTestSuite ) getWsdlModelItemByName( testSuites, testSuiteName );
	}

   public WsdlTestSuite addNewTestSuite(String name)
   {
      WsdlTestSuite testSuite = new WsdlTestSuite( this, getConfig().addNewTestSuite());
      testSuite.setName( name );
      testSuites.add( testSuite );
      fireTestSuiteAdded( testSuite );

      return testSuite;
   }

	public WsdlTestSuite cloneTestSuite(WsdlTestSuite testSuite, String name)
	{
		TestSuiteConfig testSuiteConfig = getConfig().addNewTestSuite();
		testSuiteConfig.set( testSuite.getConfig() );
		WsdlTestSuite newTestSuite = new WsdlTestSuite( this, testSuiteConfig );
		newTestSuite.setName( name );
      testSuites.add( newTestSuite );
      fireTestSuiteAdded( newTestSuite );

      return newTestSuite;
	}
	
	public boolean isCacheDefinitions()
	{
		return getSettings().getBoolean( WsdlSettings.CACHE_WSDLS );
	}
	
	public void setCacheDefinitions( boolean cacheDefinitions )
	{
		getSettings().setBoolean( WsdlSettings.CACHE_WSDLS, cacheDefinitions );
	}

	public boolean saveTo(String fileName) throws IOException
	{
		String oldPath = path;
		path = fileName;
		boolean result = save();
		if( !result )
			path = oldPath;
		
		setProjectRoot(path);
		
		return result;
	}

	public void release()
	{
		super.release();
		
		for( WsdlTestSuite testSuite : testSuites )
			testSuite.release();

		for( WsdlInterface iface : interfaces )
			iface.release();
		
		for( WsdlMockService mockService : mockServices )
			mockService.release();
	}

	public WsdlMockService cloneMockService( WsdlMockService mockService, String name )
	{
		MockServiceConfig testSuiteConfig = getConfig().addNewMockService();
		testSuiteConfig.set( mockService.getConfig() );
		WsdlMockService newMockService = new WsdlMockService( this, testSuiteConfig );
		newMockService.setName( name );
      mockServices.add( newMockService );
      fireMockServiceAdded( newMockService );

      return newMockService;
	}
	
	public WsdlMockService addNewMockService( String name )
	{
		WsdlMockService mockService = new WsdlMockService( this, getConfig().addNewMockService());
      mockService.setName( name );
      mockServices.add( mockService );
      fireMockServiceAdded( mockService );

      return mockService;
	}

	public WsdlMockService getMockServiceAt( int index )
	{
		return mockServices.get( index );
	}

	public WsdlMockService getMockServiceByName( String mockServiceName )
	{
		return (WsdlMockService) getWsdlModelItemByName( mockServices, mockServiceName );
	}

	public int getMockServiceCount()
	{
		return mockServices.size();
	}

	public void removeMockService(WsdlMockService mockService )
   {
      int ix = mockServices.indexOf( mockService );
      mockServices.remove( ix );
      
      try
      {
      	fireMockServiceRemoved( mockService );
      }
      finally
      {
      	mockService.release();
      	getConfig().removeMockService( ix );
      }
   }

	public List<TestSuite> getTestSuites()
	{
		return new ArrayList<TestSuite>( testSuites );
	}

	public List<MockService> getMockServices()
	{
		return new ArrayList<MockService>( mockServices );
	}

	public List<Interface> getInterfaces()
	{
		return new ArrayList<Interface>( interfaces );
	}

	public void reload( File file ) throws SoapUIException
	{
		this.path = file.getAbsolutePath();
		getWorkspace().reloadProject( this );
	}

   public boolean hasNature(String natureId)
   {
      Settings projectSettings = getSettings();
      String projectNature = projectSettings.getString( ProjectSettings.PROJECT_NATURE, null );
      return natureId.equals(projectNature);
   }
}