/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.actions.iface.tools.support;

import java.awt.event.ActionEvent;
import java.io.File;
import java.io.FilenameFilter;

import javax.swing.AbstractAction;
import javax.swing.Action;

import org.apache.log4j.Logger;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.actions.SoapUIPreferencesAction;
import com.eviware.soapui.impl.wsdl.actions.support.ShowOnlineHelpAction;
import com.eviware.soapui.impl.wsdl.support.wsdl.CachedWsdlLoader;
import com.eviware.soapui.model.ModelItem;
import com.eviware.soapui.model.iface.Interface;
import com.eviware.soapui.settings.ProjectSettings;
import com.eviware.soapui.support.Tools;
import com.eviware.soapui.support.UISupport;
import com.eviware.soapui.support.action.ActionList;
import com.eviware.soapui.support.action.DefaultActionList;
import com.eviware.soapui.support.types.StringToStringMap;
import com.eviware.x.form.XForm;
import com.eviware.x.form.XFormDialog;
import com.eviware.x.form.XFormDialogBuilder;
import com.eviware.x.form.XFormField;
import com.eviware.x.form.XFormTextField;

/**
 * Invokes axis 1.X WSDL2Java
 * 
 * @author Ole.Matzura
 */

public abstract class AbstractToolsAction<T extends ModelItem> extends AbstractAction
{
   private static final Logger log = Logger.getLogger(AbstractToolsAction.class);
   
	protected static final String WSDL = "WSDL";
	protected static final String CACHED_WSDL = "Use cached WSDL";
	protected static final String JAVA_ARGS = "Java Args";
	protected static final String TOOL_ARGS = "Tool Args";

	protected final T modelItem;
	protected XFormDialog dialog;
	protected String valuesSettingID;
	private XFormField useCached;

	// Configure behavior of this action:
	private boolean fixedWSDL = false;
	private Action toolsSettingsAction = new ShowIntegratedToolsSettingsAction();;

	public AbstractToolsAction(T modelItem, String name, String description)
	{
		super(name);
		this.modelItem = modelItem;
		this.valuesSettingID = this.getClass().getName() + "@values";
		if (modelItem == null)
			this.valuesSettingID += "-global";
		else
			this.valuesSettingID += "-local";

		putValue(Action.SHORT_DESCRIPTION, description);
	}

	public String getValuesSettingID()
	{
		return valuesSettingID;
	}

	public void setValuesSettingID(String valuesSettingID)
	{
		this.valuesSettingID = valuesSettingID;
	}

	/**
	 * Set this to true to not let the user edit the WSDL.
	 * 
	 * @param b
	 */
	public void setFixedWSDL(boolean b)
	{
		this.fixedWSDL = b;
	}

	public void actionPerformed(ActionEvent e)
	{
		// Could reuse the dialog in Swing, but not in Eclipse.
		// if( dialog == null )
		dialog = buildDialog();

		if (dialog == null)
		{
			try
			{
				generate(initValues(), UISupport.getToolHost());
			}
			catch (Exception e1)
			{
				UISupport.showErrorMessage(e1);
				e1.printStackTrace();
			}
		}
		else
		{
			StringToStringMap values = initValues();

			dialog.setValues(values);
			dialog.setVisible(true);
		}
	}

	protected StringToStringMap initValues()
	{
		String settingValues = modelItem == null ? SoapUI.getSettings().getString(valuesSettingID, null) : modelItem
				.getSettings().getString(valuesSettingID, null);

		StringToStringMap result = settingValues == null ? new StringToStringMap() : StringToStringMap
				.fromXml(settingValues);

		if (modelItem instanceof Interface)
		{
			initWSDL(result, (Interface) modelItem);
		}

		if (dialog != null && modelItem != null)
		{
			String projectRoot = modelItem.getSettings().getString( ProjectSettings.PROJECT_ROOT, null);
			if( projectRoot != null ) 
				dialog.setFormFieldProperty(ProjectSettings.PROJECT_ROOT, projectRoot);
		}

		return result;
	}

	protected XFormDialog buildDialog()
	{
		return null;
	}

	protected void addWSDLFields(XForm mainForm)
	{
		if (!fixedWSDL)
		{
			XFormTextField tf = mainForm.addTextField(WSDL, "url to wsdl", XForm.FieldType.URL);

			if (modelItem instanceof Interface)
			{
				useCached = mainForm.addCheckBox(CACHED_WSDL, null);
				useCached.addComponentEnabler(tf, "false");
			}
		}
		else
		{
			if (modelItem instanceof Interface)
			{
				useCached = mainForm.addCheckBox(CACHED_WSDL, null);
			}
		}
	}

	protected void initWSDL(StringToStringMap values, Interface iface)
	{
		boolean cached = iface.isCached();
      if (useCached != null)
			useCached.setEnabled(cached);

		if (!values.containsKey(CACHED_WSDL))
			values.put(CACHED_WSDL, Boolean.toString(cached));

		if (values.getBoolean(CACHED_WSDL) || !values.hasValue(WSDL))
			values.put(WSDL, iface.getDefinition());
	}

	protected abstract void generate(StringToStringMap values, ToolHost toolHost) throws Exception;

	public void run(ToolHost toolHost) throws Exception
	{
		generate(initValues(), toolHost);
	}

	/**
	 * To be overridden..
	 */

	public void onClose()
	{
		if (dialog == null)
			return;

		if (modelItem == null)
		{
			SoapUI.getSettings().setString(valuesSettingID, dialog.getValues().toXml());
		}
		else
		{
			modelItem.getSettings().setString(valuesSettingID, dialog.getValues().toXml());
		}
	}

	protected String getWsdlUrl(StringToStringMap values)
	{
		String wsdl = values.get(WSDL);
		boolean useCached = values.getBoolean(CACHED_WSDL);

		if (wsdl == null && !useCached && modelItem instanceof Interface)
			return ((Interface) modelItem).getDefinition();

		Interface iface = (Interface) modelItem;
		if (useCached && iface.isCached())
		{
			try
			{
				File tempFile = File.createTempFile("tempdir", null);
				String path = tempFile.getAbsolutePath();
				tempFile.delete();
				CachedWsdlLoader loader = (CachedWsdlLoader) iface.createWsdlLoader();
				wsdl = loader.saveDefinition(path);
			}
			catch (Exception e)
			{
				e.printStackTrace();
			}
		}

		return wsdl;
	}

	protected String buildClasspath(File jarDir)
	{
		String[] jars = jarDir.list(new FilenameFilter()
		{

			public boolean accept(File dir, String name)
			{
				return name.endsWith(".jar");
			}
		});

		StringBuilder classpath = new StringBuilder();

		for (int c = 0; c < jars.length; c++)
		{
			if (c > 0)
				classpath.append(File.pathSeparatorChar);

			classpath.append(jars[c]);
		}
		return classpath.toString();
	}

	protected ActionList buildDefaultActions(String helpUrl)
	{
		ActionList actions = new DefaultActionList("Actions");

		if (helpUrl != null)
		{
			actions.addAction(new ShowOnlineHelpAction(helpUrl));
			actions.addSeparator();
		}

		actions.addAction(createRunOption());
		actions.addAction(new CloseAction());
		
		if( toolsSettingsAction != null)
			actions.addAction(toolsSettingsAction);

		return actions;
	}
	
	public Action getToolsSettingsAction()
	{
		return toolsSettingsAction;
	}

	public void setToolsSettingsAction(Action toolsSettingsAction)
	{
		this.toolsSettingsAction = toolsSettingsAction;
	}

	protected Action createRunOption()
	{
		return new GenerateAction();
	}

	protected String getDefinition()
	{
		if (modelItem == null)
			return "";

		String definition = ((Interface) modelItem).getDefinition();
		if (definition.startsWith("file:"))
			definition = definition.substring(5);

		return definition;
	}

	protected void addJavaArgs(StringToStringMap values, ArgumentBuilder builder)
	{
		String[] javaArgs = Tools.tokenizeArgs(values.get(JAVA_ARGS));
		if (javaArgs != null)
			builder.addArgs(javaArgs);
	}

	protected void addToolArgs(StringToStringMap values, ArgumentBuilder builder)
	{
		String[] toolArgs = Tools.tokenizeArgs(values.get(TOOL_ARGS));
		if (toolArgs != null)
			builder.addArgs(toolArgs);
	}

	protected XForm buildArgsForm(XFormDialogBuilder builder, boolean addJavaArgs, String toolName)
	{
		XForm argsForm = builder.createForm("Custom Args");
		if (addJavaArgs)
			argsForm.addTextField(JAVA_ARGS, "additional arguments to java", XForm.FieldType.TEXT);

		argsForm.addTextField(TOOL_ARGS, "additional arguments to " + toolName, XForm.FieldType.TEXT);
		return argsForm;
	}

	public static final class ShowIntegratedToolsSettingsAction extends AbstractAction
	{
		public ShowIntegratedToolsSettingsAction()
		{
			super("Tools");
		}

		public void actionPerformed(ActionEvent e)
		{
			SoapUIPreferencesAction.getInstance().show(SoapUIPreferencesAction.INTEGRATED_TOOLS);
		}
	}

	protected final class CloseAction extends AbstractAction
	{
		public CloseAction()
		{
			super("Close");
		}

		public void actionPerformed(ActionEvent e)
		{
			closeDialog();
		}
	}

	public void closeDialog()
	{
		onClose();
		if (dialog != null)
			dialog.setVisible(false);
	}

	protected final class GenerateAction extends AbstractAction
	{
		public GenerateAction()
		{
			super("Generate");
		}

		public void actionPerformed(ActionEvent e)
		{
			try
			{
				if (dialog.validate())
				{
					generate(dialog.getValues(), UISupport.getToolHost());
				}
			}
			catch (Exception e1)
			{
				UISupport.showErrorMessage(e1);
			}
		}
	}
}
