/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.loadtest;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.apache.log4j.Logger;

import com.eviware.soapui.config.LoadStrategyConfig;
import com.eviware.soapui.config.LoadTestAssertionConfig;
import com.eviware.soapui.config.LoadTestConfig;
import com.eviware.soapui.config.LoadTestLimitTypesConfig;
import com.eviware.soapui.config.LoadTestLimitTypesConfig.Enum;
import com.eviware.soapui.impl.wsdl.AbstractWsdlModelItem;
import com.eviware.soapui.impl.wsdl.actions.loadtest.CloneLoadTestAction;
import com.eviware.soapui.impl.wsdl.actions.loadtest.RemoveLoadTestAction;
import com.eviware.soapui.impl.wsdl.actions.loadtest.RenameLoadTestAction;
import com.eviware.soapui.impl.wsdl.actions.support.ShowOnlineHelpAction;
import com.eviware.soapui.impl.wsdl.loadtest.assertions.AbstractLoadTestAssertion;
import com.eviware.soapui.impl.wsdl.loadtest.assertions.LoadTestAssertionRegistry;
import com.eviware.soapui.impl.wsdl.loadtest.data.LoadTestStatistics;
import com.eviware.soapui.impl.wsdl.loadtest.log.LoadTestLog;
import com.eviware.soapui.impl.wsdl.loadtest.log.LoadTestLogErrorEntry;
import com.eviware.soapui.impl.wsdl.loadtest.strategy.BurstLoadStrategy;
import com.eviware.soapui.impl.wsdl.loadtest.strategy.LoadStrategy;
import com.eviware.soapui.impl.wsdl.loadtest.strategy.LoadStrategyFactory;
import com.eviware.soapui.impl.wsdl.loadtest.strategy.LoadStrategyRegistry;
import com.eviware.soapui.impl.wsdl.loadtest.strategy.SimpleLoadStrategy;
import com.eviware.soapui.impl.wsdl.support.Configurable;
import com.eviware.soapui.impl.wsdl.support.HelpUrls;
import com.eviware.soapui.impl.wsdl.testcase.WsdlTestCase;
import com.eviware.soapui.model.support.LoadTestRunListenerAdapter;
import com.eviware.soapui.model.testsuite.LoadTest;
import com.eviware.soapui.model.testsuite.LoadTestRunContext;
import com.eviware.soapui.model.testsuite.LoadTestRunListener;
import com.eviware.soapui.model.testsuite.LoadTestRunner;
import com.eviware.soapui.model.testsuite.TestRunContext;
import com.eviware.soapui.model.testsuite.TestRunner;
import com.eviware.soapui.model.testsuite.TestStepResult;
import com.eviware.soapui.model.testsuite.LoadTestRunner.Status;
import com.eviware.soapui.support.action.ActionSupport;

/**
 * TestCase implementation for LoadTests
 *  
 * @todo add assertionFailed event to LoadTestListener
 * @todo create and return LoadTestAssertionResult from load-test assertions 
 *  
 * @author Ole.Matzura
 */

public class WsdlLoadTest extends AbstractWsdlModelItem<LoadTestConfig> implements LoadTest
{
	public final static String THREADCOUNT_PROPERTY = WsdlLoadTest.class.getName() + "@threadcount";
	public final static String STARTDELAY_PROPERTY = WsdlLoadTest.class.getName() + "@startdelay";
	public final static String TESTLIMIT_PROPERTY = WsdlLoadTest.class.getName() + "@testlimit";
	public final static String LIMITTYPE_PROPERRY = WsdlLoadTest.class.getName() + "@limittype";
	public final static String SAMPLEINTERVAL_PROPERRY = WsdlLoadTest.class.getName() + "@sample-interval";

   private final static Logger logger = Logger.getLogger( WsdlLoadTest.class );
   private InternalTestRunListener internalTestRunListener = new InternalTestRunListener();
   
	private WsdlTestCase testCase;
	private LoadTestStatistics statisticsModel;
	private LoadStrategy loadStrategy = new BurstLoadStrategy();
	private LoadTestLog loadTestLog;
	
	private LoadStrategyConfigurationChangeListener loadStrategyListener = new LoadStrategyConfigurationChangeListener();
	private List<LoadTestAssertion> assertions = new ArrayList<LoadTestAssertion>();
	private ConfigurationChangePropertyListener configurationChangeListener = new ConfigurationChangePropertyListener();
	private Set<LoadTestListener> listeners = new HashSet<LoadTestListener>();
	private Set<LoadTestRunListener> runListeners = new HashSet<LoadTestRunListener>();
	private WsdlLoadTestRunner runner;
   
   public WsdlLoadTest(WsdlTestCase testCase, LoadTestConfig config )
   {
   	super( config, testCase, "/loadTest.gif" );
   	
      this.testCase = testCase;
		
		if( getConfig().getThreadCount() < 1 )
			getConfig().setThreadCount( 5 );
		
		if( getConfig().getLimitType() == null )
		{
			getConfig().setLimitType( LoadTestLimitTypesConfig.TIME );
			getConfig().setTestLimit( 60 );
		}	
      
      addAction( new RenameLoadTestAction( this ));
      addAction( new RemoveLoadTestAction( this ));
      addAction( ActionSupport.SEPARATOR_ACTION );
      addAction( new CloneLoadTestAction( this ));
      addAction( ActionSupport.SEPARATOR_ACTION );
      addAction( new ShowOnlineHelpAction( HelpUrls.LOADTEST_HELP_URL ));
      
      addLoadTestRunListener( internalTestRunListener );
      
      LoadStrategyConfig ls = getConfig().getLoadStrategy();
      if( ls == null )
      {
      	ls = getConfig().addNewLoadStrategy();
      	ls.setType( SimpleLoadStrategy.STRATEGY_TYPE );
      }
      
      LoadStrategyFactory factory = LoadStrategyRegistry.getInstance().getFactory( ls.getType() );
      if( factory == null )
      {
      	ls.setType( SimpleLoadStrategy.STRATEGY_TYPE );
      	factory = LoadStrategyRegistry.getInstance().getFactory( ls.getType() );
      }
      
      loadStrategy = factory.build( ls.getConfig() );
      loadStrategy.addConfigurationChangeListener( loadStrategyListener );
      
      addLoadTestRunListener( loadStrategy );
      
      statisticsModel = new LoadTestStatistics( this );
      
      if( getConfig().xgetSampleInterval() == null )
      	setSampleInterval( LoadTestStatistics.DEFAULT_SAMPLE_INTERVAL );
      
      statisticsModel.setUpdateFrequency( getSampleInterval() );
      
      List<LoadTestAssertionConfig> assertionList = getConfig().getAssertionList();
      for( LoadTestAssertionConfig assertionConfig : assertionList )
      {
      	AbstractLoadTestAssertion assertion = LoadTestAssertionRegistry.buildAssertion( assertionConfig, this );
      	if( assertion != null )
      	{
      		assertions.add( assertion);
      		assertion.addPropertyChangeListener( LoadTestAssertion.CONFIGURATION_PROPERTY, configurationChangeListener );
      	}
      	else
      	{
      		logger.warn( "Failed to build LoadTestAssertion from getConfig() [" + assertionConfig + "]" );
      	}
      }
      
      if( getConfig().xgetResetStatisticsOnThreadCountChange() == null )
      	getConfig().setResetStatisticsOnThreadCountChange( true );

      if( getConfig().xgetCalculateTPSOnTimePassed() == null )
      	getConfig().setCalculateTPSOnTimePassed( false );
      
      loadTestLog = new LoadTestLog( this );
   }

   public LoadTestStatistics getStatisticsModel()
   {
   	return statisticsModel;
   }
   
   public long getThreadCount()
   {
   	return getConfig().getThreadCount();
   }
   
   public void setThreadCount( long threadCount )
   {
   	if( threadCount < 1 || threadCount == getThreadCount() )
   		return;
   	
   	long oldCount = getThreadCount();
   	getConfig().setThreadCount( (int) threadCount );
   	notifyPropertyChanged( THREADCOUNT_PROPERTY, oldCount, threadCount );
   }
   
   public boolean getResetStatisticsOnThreadCountChange()
   {
   	return getConfig().getResetStatisticsOnThreadCountChange();
   }
   
   public void setResetStatisticsOnThreadCountChange( boolean value )
   {
   	getConfig().setResetStatisticsOnThreadCountChange( value );
   }   

   public boolean getCalculateTPSOnTimePassed()
   {
   	return getConfig().getCalculateTPSOnTimePassed();
   }
   
   public void setCalculateTPSOnTimePassed( boolean value )
   {
   	getConfig().setCalculateTPSOnTimePassed( value );
   }   

   public int getStartDelay()
   {
   	return getConfig().getStartDelay();
   }
   
   public void setStartDelay( int startDelay )
   {
   	if( startDelay < 0 )
   		return;
   	
   	int oldDelay = getStartDelay();
   	getConfig().setStartDelay( startDelay );
   	notifyPropertyChanged( STARTDELAY_PROPERTY, oldDelay, startDelay );
   }

   public long getTestLimit()
   {
   	return getConfig().getTestLimit();
   }
   
   public void setTestLimit( long testLimit )
   {
   	if( testLimit < 0 )
   		return;
   	
   	long oldLimit = getTestLimit();
   	getConfig().setTestLimit( testLimit );
   	notifyPropertyChanged( TESTLIMIT_PROPERTY, oldLimit, testLimit );
   }
   
   public long getSampleInterval()
   {
   	return getConfig().getSampleInterval();
   }
   
   public void setSampleInterval( int sampleInterval )
   {
   	if( sampleInterval < 0 )
   		return;
   	
   	long oldInterval = getSampleInterval();
   	getConfig().setSampleInterval( sampleInterval );
   	
   	statisticsModel.setUpdateFrequency( sampleInterval );
   	
   	notifyPropertyChanged( TESTLIMIT_PROPERTY, oldInterval, sampleInterval );
   }

   public Enum getLimitType()
   {
   	return getConfig().getLimitType();
   }
   
   public void setLimitType( Enum limitType )
   {
   	if( limitType == null )
   		return;
   	
   	Enum oldType = getLimitType();
   	getConfig().setLimitType( limitType );
   	notifyPropertyChanged( LIMITTYPE_PROPERRY, oldType, limitType );
   }
   
   public WsdlTestCase getTestCase()
	{
		return testCase;
	}

   public synchronized LoadTestRunner run() 
   {
   	if( runner != null && runner.getStatus() == Status.RUNNING )
   		return null;
   	
   	runner = new WsdlLoadTestRunner( this );
   	runner.start();
   	return runner;
	}

	private class InternalTestRunListener extends LoadTestRunListenerAdapter
	{
		public void afterTestCase(LoadTestRunner loadTestRunner, LoadTestRunContext context, TestRunner testRunner, TestRunContext runContext)
		{
			if( !assertions.isEmpty() )
			{
				for( LoadTestAssertion assertion : assertions )
				{
					String error = assertion.assertResults( loadTestRunner, context, testRunner, runContext );
					if( error != null )
					{
						loadTestLog.addEntry( new LoadTestLogErrorEntry( assertion.getName(), error, assertion.getIcon() ));
						statisticsModel.addError( LoadTestStatistics.TOTAL );
					}	
				}
			}				
		}

		public void afterTestStep(LoadTestRunner loadTestRunner, LoadTestRunContext context, TestRunner testRunner, TestRunContext runContext, TestStepResult result)
		{
			if( !assertions.isEmpty() )
			{
				boolean added = false;
				
				for( LoadTestAssertion assertion : assertions )
				{
					String error = assertion.assertResult( loadTestRunner, context, result, testRunner, runContext );
					if( error != null )
					{
						int indexOfTestStep = testRunner.getTestCase().getIndexOfTestStep( result.getTestStep() );
						
						loadTestLog.addEntry( new LoadTestLogErrorEntry( assertion.getName(), error, result, assertion.getIcon() ));
						statisticsModel.addError( indexOfTestStep);
						
						added = true;
					}	
				}

				// always discard result if there were no errors
				if( !added )
				{
					result.discard();
				}
			}				
			else result.discard();
		}
	}

	public LoadStrategy getLoadStrategy()
	{
		return loadStrategy;
	}
	
	public void setLoadStrategy( LoadStrategy loadStrategy )
	{
		this.loadStrategy.removeConfigurationChangeListener( loadStrategyListener );
		removeLoadTestRunListener( this.loadStrategy );
		
		this.loadStrategy = loadStrategy;
		this.loadStrategy.addConfigurationChangeListener( loadStrategyListener );
		addLoadTestRunListener( this.loadStrategy );
		
		getConfig().getLoadStrategy().setType( loadStrategy.getType() );
		getConfig().getLoadStrategy().setConfig( loadStrategy.getConfig() );
	}

	private class LoadStrategyConfigurationChangeListener implements PropertyChangeListener
	{
		public void propertyChange(PropertyChangeEvent evt)
		{
			getConfig().getLoadStrategy().setConfig( loadStrategy.getConfig() );
		}
	}

	public LoadTestAssertion addAssertion( String type, String targetStep, boolean showConfig )
	{
		LoadTestAssertion assertion = LoadTestAssertionRegistry.createAssertion( type, this );
		assertion.setTargetStep( targetStep );
		
		if( assertion instanceof Configurable && showConfig )
		{
			if( !((Configurable)assertion).configure() )
				return null;
		}
		
		assertions.add( assertion );
		
		getConfig().addNewAssertion().set( assertion.getConfiguration() );
		assertion.addPropertyChangeListener( LoadTestAssertion.CONFIGURATION_PROPERTY, configurationChangeListener );
		fireAssertionAdded( assertion );
		
		return assertion;
	}
	
	public void removeAssertion( LoadTestAssertion assertion)
	{
		int ix = assertions.indexOf( assertion );
		if( ix >= 0 )
		{
			try
			{
				assertions.remove( ix );
				fireAssertionRemoved(assertion);
			}
			finally
			{
				assertion.removePropertyChangeListener( configurationChangeListener );
				assertion.release();
				getConfig().removeAssertion( ix );
			}
		}
	}

	private void fireAssertionRemoved(LoadTestAssertion assertion)
	{
		if( !listeners.isEmpty() )
		{
			LoadTestListener[] l = listeners.toArray( new LoadTestListener[listeners.size()] );
			for( LoadTestListener listener : l )
			{
				listener.assertionRemoved( assertion );
			}
		}
	}
	
	private void fireAssertionAdded(LoadTestAssertion assertion)
	{
		if( !listeners.isEmpty() )
		{
			LoadTestListener[] l = listeners.toArray( new LoadTestListener[listeners.size()] );
			for( LoadTestListener listener : l )
			{
				listener.assertionAdded( assertion );
			}
		}
	}

	public int getAssertionCount()
	{
		return assertions.size();
	}
	
	public LoadTestAssertion getAssertionAt( int index )
	{
		return assertions.get( index );
	}
	
	private class ConfigurationChangePropertyListener implements PropertyChangeListener
	{
		public void propertyChange(PropertyChangeEvent evt)
		{
			 int ix = assertions.indexOf( evt.getSource() );
			 if( ix >= 0 )
			 {
				 getConfig().getAssertionArray( ix ).set( assertions.get( ix ).getConfiguration() );
			 }
		}
	}

	public LoadTestLog getLoadTestLog()
	{
		return loadTestLog;
	}

	public List<LoadTestAssertion> getAssertionList()
	{
		return assertions;
	}
	
	public void addLoadTestListener( LoadTestListener listener )
	{
		listeners.add( listener );
	}

	public void removeLoadTestListener( LoadTestListener listener )
	{
		listeners.remove( listener );
	}

	public void addLoadTestRunListener(LoadTestRunListener listener)
	{
		runListeners.add( listener );
	}

	public void removeLoadTestRunListener(LoadTestRunListener listener)
	{
		runListeners.remove( listener );
	}
	
	public LoadTestRunListener [] getLoadTestRunListeners()
	{
		return runListeners.toArray( new LoadTestRunListener[runListeners.size()] );
	}

	/**
	 * Release internal objects so they can remove listeners
	 */
	
	public void release()
	{
		super.release();
		
		statisticsModel.release();
		loadTestLog.release();
		
		for( LoadTestAssertion assertion : assertions )
			assertion.release();
	}

	public boolean isRunning()
	{
		return runner != null && runner.getStatus() == LoadTestRunner.Status.RUNNING;
	}

	public WsdlLoadTestRunner getRunner()
	{
		return runner;
	}
}