/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.panels.request.components.editor;

import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.AbstractAction;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.JToggleButton;
import javax.swing.SwingUtilities;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import com.eviware.soapui.support.UISupport;
import com.eviware.soapui.support.components.JXToolBar;
import com.eviware.soapui.support.components.VTextIcon;
import com.eviware.soapui.support.components.VerticalTabbedPaneUI;

@SuppressWarnings("serial")
public class XmlEditor extends JPanel implements PropertyChangeListener, XmlLocationListener
{
	private static final float DEFAULT_DIVIDER_LOCATION = 0.7F;
	private JTabbedPane inputTabs;
	private List<XmlEditorView> views = new ArrayList<XmlEditorView>();
	private XmlEditorView currentView;
	private XmlDocument xmlDocument;
	private List<XmlInspector> inspectors = new ArrayList<XmlInspector>();
	protected JSplitPane mainSplit;
	private JXToolBar inspectToolbar;
	private Map<XmlInspector,JToggleButton> inspectorButtons = new HashMap<XmlInspector,JToggleButton>();
	private XmlInspector currentInspector;
	private JPanel inspectorPanel;
	private int lastDividerLocation = 0;

	public XmlEditor( XmlDocument xmlDocument )
	{
		super( new BorderLayout() );
		this.xmlDocument = xmlDocument;
		
		setBackground(Color.LIGHT_GRAY);
	   inputTabs = new JTabbedPane(JTabbedPane.LEFT, JTabbedPane.SCROLL_TAB_LAYOUT);
   	inputTabs.setUI(new VerticalTabbedPaneUI());
	   
		inputTabs.setFont(inputTabs.getFont().deriveFont(8));
		
		inputTabs.addChangeListener( new ChangeListener() 
		{
			public void stateChanged( ChangeEvent e )
			{
				int currentViewIndex = views.indexOf( currentView );
				
				if( currentView != null )
				{
					if( inputTabs.getSelectedIndex() == currentViewIndex)
						return;
					
					if( !currentView.deactivate() )
					{
						inputTabs.setSelectedIndex( currentViewIndex );
						return;
					}
				}
				
				XmlEditorView previousView = currentView;
				int selectedIndex = inputTabs.getSelectedIndex();
				if( selectedIndex == -1 )
				{
					currentView = null;
					return;
				}
				
				currentView = views.get( selectedIndex );
				
				if( currentView != null && !currentView.activate( previousView == null ? null : previousView.getLocation() ))
				{
					inputTabs.setSelectedIndex( currentViewIndex );
					if( currentViewIndex == -1 )
						return;
				}
				
				if( !currentView.isInspectable() && currentInspector != null )
					lastDividerLocation = mainSplit.getDividerLocation();
				
				inspectorPanel.setVisible( currentView.isInspectable() && currentInspector != null );
				inspectToolbar.setVisible( currentView.isInspectable()  );
				
				if( currentView.isInspectable() && currentInspector != null )
				{
					if( lastDividerLocation == 0 )
						mainSplit.setDividerLocation( DEFAULT_DIVIDER_LOCATION );
					else
						mainSplit.setDividerLocation( lastDividerLocation );
				}
				
				SwingUtilities.invokeLater( new Runnable() {

					public void run()
					{
						if( currentView != null )
							currentView.getComponent().requestFocus();
					}} );
				
			}
		} );
		
		inspectorPanel = new JPanel(new CardLayout());
		inspectorPanel.setVisible( false );
		
		mainSplit = new JSplitPane( JSplitPane.VERTICAL_SPLIT, inputTabs, inspectorPanel );
		mainSplit.setDividerSize( 10 );
		mainSplit.setBorder( null );
		mainSplit.setOneTouchExpandable( false );
		
		add(mainSplit, BorderLayout.CENTER);
		add( createInspectButtons(), BorderLayout.PAGE_END  );
		
		mainSplit.setResizeWeight( 0.8 );
	}
	
	private JComponent createInspectButtons()
	{
		inspectToolbar = UISupport.createToolbar();
		inspectToolbar.addSpace( 10 );
		inspectToolbar.setBackground( Color.WHITE );
		inspectToolbar.setOpaque( true );
		return inspectToolbar;
	}

	public void addInspector( XmlInspector inspector )
	{
		if( inspectors.size() > 0 )
		{
			inspectToolbar.addSeparator();
		}
		
		inspectors.add( inspector );
		inspectorPanel.add( inspector.getComponent(), inspector.getClass().getName() );

		JToggleButton button = new JToggleButton( new SelectInspectorAction( inspector ));
		
		button.setToolTipText( inspector.getDescription() );
		button.setOpaque( true );
		
		inspectorButtons.put( inspector, button );
		inspectToolbar.add( button );
		inspectToolbar.addRelatedGap();
	}
	
	public void addEditorView( XmlEditorView editorView )
	{
		views.add( editorView );
		
		inputTabs.addTab( null, new VTextIcon(inputTabs, editorView.getTitle(), VTextIcon.ROTATE_LEFT ),
				editorView.getComponent() );

		editorView.addPropertyChangeListener( XmlEditorView.TITLE_PROPERTY, this );
		editorView.addLocationListener( this );
		
		editorView.setXmlDocument( xmlDocument );
	}

	public void propertyChange(PropertyChangeEvent evt)
	{
		int ix = views.indexOf( evt.getSource() );
		if( ix == -1 )
			return;
		
		inputTabs.setTitleAt( ix, (String) evt.getNewValue() );
	}
	
	public void selectView( int viewIndex )
	{
		inputTabs.setSelectedIndex( viewIndex );
	}
	
	public void requestFocus()
	{
		if( currentView != null )
			currentView.getComponent().requestFocus();
	}

	public final XmlDocument getXmlDocument()
	{
		return xmlDocument;
	}
	
	public boolean saveDocument( boolean validate )
	{
	   return currentView == null ? true : currentView.saveDocument( validate );
	}
	
	public boolean hasFocus()
	{
		return currentView == null ? false : currentView.getComponent().hasFocus();
	}

	public final void setXmlDocument(XmlDocument xmlDocument)
	{
		this.xmlDocument = xmlDocument;
		
		for( XmlEditorView view : views )
		{
			view.setXmlDocument( xmlDocument );
		}
	}

	public final XmlEditorView getCurrentView()
	{
		return currentView;
	}

	public final JTabbedPane getInputTabs()
	{
		return inputTabs;
	}

	public final List<XmlEditorView> getViews()
	{
		return views;
	}
	
	public XmlEditorView getView( String viewId )
	{
		for( XmlEditorView view : views )
		{
			if( view.getViewId().equals( viewId ))
				return view;
		}
		
		return null;
	}
	
	public XmlInspector getInspector( String inspectorId  )
	{
		for( XmlInspector inspector : inspectors )
		{
			if( inspector.getInspectorId().equals( inspectorId ))
				return inspector;
		}
		
		return null;
	}

	public void locationChanged(XmlLocation location)
	{
		if( location != null )
		{
			for( XmlInspector inspector : inspectors )
			{
				inspector.locationChanged( location );
			}
		}
	}
	
	public void setEditable( boolean enabled )
	{
		for( XmlEditorView view : views )
		{
			view.setEditable( enabled );
		}
	}
	
	public class SelectInspectorAction extends AbstractAction implements PropertyChangeListener
	{
		private final XmlInspector inspector;

		public SelectInspectorAction( XmlInspector inspector )
		{
			super( inspector.getTitle());
			this.inspector = inspector;
			inspector.addPropertyChangeListener( this );
		}

		public void actionPerformed( ActionEvent arg0 )
		{
			JToggleButton button = inspectorButtons.get( inspector );
			if( !button.isSelected() )
			{
				currentInspector = null;
				button.setBackground( inspectToolbar.getBackground() );
				lastDividerLocation = mainSplit.getDividerLocation();
				inspectorPanel.setVisible( false );
			}
			else
			{
				if( currentInspector != null )
					inspectorButtons.get( currentInspector ).setSelected( false );
				
				currentInspector = inspector;
				button.setBackground( Color.WHITE );
				
				if( !inspectorPanel.isVisible() )
				{
					inspectorPanel.setVisible( true );
					if( lastDividerLocation == 0 )
						mainSplit.setDividerLocation( DEFAULT_DIVIDER_LOCATION );
					else
						mainSplit.setDividerLocation( lastDividerLocation );
				}
				
				CardLayout cards = ( CardLayout ) inspectorPanel.getLayout();
				cards.show( inspectorPanel, inspector.getClass().getName() );
			}
		}

		public void propertyChange( PropertyChangeEvent evt )
		{
			if( evt.getPropertyName().equals( XmlInspector.TITLE_PROPERTY ))
				putValue( AbstractAction.NAME, evt.getNewValue() );
			else if( evt.getPropertyName().equals( XmlInspector.DESCRIPTION_PROPERTY ))
				putValue( AbstractAction.SHORT_DESCRIPTION, evt.getNewValue() );
			else if( evt.getPropertyName().equals( XmlInspector.ENABLED_PROPERTY ))
			{
				boolean enable = ((Boolean)evt.getNewValue()).booleanValue();
				setEnabled( enable );
				
				if( !enable && currentInspector == inspector )
				{
					inspectorButtons.get( currentInspector ).setSelected( false );
				}
			}
		}
	}
}
