/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.submit.filters;

import org.apache.log4j.Logger;
import org.apache.log4j.Priority;
import org.apache.xmlbeans.XmlException;
import org.apache.xmlbeans.XmlObject;

import com.eviware.soapui.impl.wsdl.WsdlRequest;
import com.eviware.soapui.impl.wsdl.submit.RequestFilter;
import com.eviware.soapui.impl.wsdl.submit.transports.http.BaseHttpRequestTransport;
import com.eviware.soapui.model.iface.SubmitContext;
import com.eviware.soapui.model.testsuite.TestRunContext;
import com.eviware.soapui.support.xml.XmlUtils;

/**
 * RequestFilter that expands properties in request content
 * 
 * @author Ole.Matzura
 */

public class PropertyExpansionRequestFilter implements RequestFilter
{
	private static final char PROPERTY_SEPARATOR = '#';
	private final static Logger log = Logger.getLogger(PropertyExpansionRequestFilter.class);
	
	public void filterRequest(SubmitContext context, WsdlRequest wsdlRequest)
	{
		String content = (String) context.getProperty( BaseHttpRequestTransport.REQUEST_CONTENT );
		if( content == null )
		{
			log.warn( "Missing request content in context, skipping property expansion" );
		}
		else
		{
			content = expandProperties(context, content);
			if( content != null )
				context.setProperty( BaseHttpRequestTransport.REQUEST_CONTENT, content );
		}
	}

	public static String expandProperties(SubmitContext context, String content)
	{
		int ix = content.indexOf( "${" );
		if( ix == -1 )
			return content;
		
		StringBuffer buf = new StringBuffer();
		int lastIx = 0;
		while( ix != -1 )
		{
			buf.append( content.substring( lastIx, ix ));
			
			int ix2 = content.indexOf( '}', ix+2 );
			if( ix2 == -1 )
				break;

			int ix3 = content.lastIndexOf( "${", ix2 );
			if( ix3 != ix )
			{
				buf.append( content.substring( ix, ix3 ));
				ix = ix3;
			}
			
			String propertyName = content.substring( ix+2, ix2 );
			Object property = context.getProperty( propertyName );
			if( property != null )
			{
				buf.append( property.toString() );
			}
			else if( context instanceof TestRunContext )
			{
				int sepIx = propertyName.indexOf( PROPERTY_SEPARATOR );
				if( sepIx > 0 )
				{
					String step = propertyName.substring( 0, sepIx );
					String name = propertyName.substring( sepIx+1 );
					
					sepIx = name.indexOf( PROPERTY_SEPARATOR );
					if( sepIx != -1 )
					{
						String xpath = name.substring( sepIx+1 );
						name = name.substring( 0, sepIx );
						
						if( step.length() == 0 )
							property = ((TestRunContext)context).getProperty( name);
						else
							property = ((TestRunContext)context).getProperty( step, name);
						
						if( property != null )
						{
							property = extractXPathPropertyValue( property, xpath );
						}
					}
					else
					{
						if( step.length() == 0 )
							property = ((TestRunContext)context).getProperty( name);
						else
							property = ((TestRunContext)context).getProperty( step, name);
					}
					
					if( property != null )
					{
						buf.append( property.toString() );
					}
				}
			}
			else if( propertyName.charAt( 0 ) == PROPERTY_SEPARATOR  )
			{
				int sepIx = propertyName.indexOf( PROPERTY_SEPARATOR, 1 );
				if( sepIx > 0 )
				{
					String xpath = propertyName.substring( sepIx+1 );
					propertyName = propertyName.substring( 1, sepIx );
					
					property = context.getProperty( propertyName );
					if( property != null )
					{
						property = extractXPathPropertyValue( property, xpath );
					}
				}
				else
				{
					property = context.getProperty( propertyName.substring( 1 ) );
				}
				
				if( property != null )
				{
					buf.append( property.toString() );
				}
			}
				
			if( property == null )
			{
				if( log.isEnabledFor( Priority.WARN ))
					log.warn( "Missing property value for [" + propertyName + "]" );
				
			   buf.append( "${" ).append( propertyName ).append( '}' );
			}
			
			lastIx = ix2+1;
			ix = content.indexOf( "${", lastIx );
		}
		
		if( lastIx < content.length() )
			buf.append( content.substring( lastIx ));
		
		return buf.toString();
	}

	public static String extractXPathPropertyValue( Object property, String xpath )
	{
		try
		{
			XmlObject xmlObject = XmlObject.Factory.parse(  property.toString() );
			String ns = XmlUtils.declareXPathNamespaces( xmlObject );
			XmlObject[] paths = xmlObject.selectPath( ns + xpath );
			if( paths.length > 0 )
				return XmlUtils.getNodeValue( paths[0].getDomNode() );
		}
		catch( XmlException e )
		{
			e.printStackTrace();
		}
		
		return null;
	}
}
