/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.submit.transports.http;

import java.util.ArrayList;
import java.util.List;

import javax.activation.DataHandler;
import javax.mail.MessagingException;
import javax.mail.internet.MimeBodyPart;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.MimeMultipart;
import javax.mail.internet.PreencodedMimeBodyPart;

import org.apache.commons.httpclient.Header;
import org.apache.commons.httpclient.HostConfiguration;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpState;
import org.apache.commons.httpclient.methods.ByteArrayRequestEntity;
import org.apache.log4j.Logger;

import com.eviware.soapui.impl.wsdl.WsdlOperation;
import com.eviware.soapui.impl.wsdl.WsdlRequest;
import com.eviware.soapui.impl.wsdl.submit.RequestFilter;
import com.eviware.soapui.impl.wsdl.submit.filters.PropertyExpansionRequestFilter;
import com.eviware.soapui.impl.wsdl.support.MessageXmlObject;
import com.eviware.soapui.impl.wsdl.support.MessageXmlPart;
import com.eviware.soapui.impl.wsdl.support.http.HttpClientSupport;
import com.eviware.soapui.model.iface.Response;
import com.eviware.soapui.model.iface.SubmitContext;
import com.eviware.soapui.model.settings.Settings;
import com.eviware.soapui.settings.HttpSettings;
import com.eviware.soapui.support.StringUtils;
import com.eviware.soapui.support.types.StringToStringMap;

/**
 * HTTP transport that uses HttpClient to send/receive SOAP messages
 * 
 * @author Ole.Matzura
 */

public class HttpClientRequestTransport implements BaseHttpRequestTransport
{
	private List<RequestFilter> filters = new ArrayList<RequestFilter>();
	private final static Logger log = Logger.getLogger(HttpClientRequestTransport.class);
	
	public void addRequestFilter(RequestFilter filter)
	{
		filters.add( filter );
	}

	public void removeRequestFilter(RequestFilter filter)
	{
		filters.remove( filter );
	}

	public void abortRequest( SubmitContext submitContext )
	{
		TimeablePostMethod postMethod = (TimeablePostMethod) submitContext.getProperty( POST_METHOD );
		postMethod.abort();
	}

	public Response sendRequest( SubmitContext submitContext, WsdlRequest wsdlRequest ) throws Exception
	{
		HttpClient httpClient = HttpClientSupport.getHttpClient();
		TimeablePostMethod postMethod = new TimeablePostMethod();
		boolean createdState = false;
		
		HttpState httpState = (HttpState) submitContext.getProperty(SubmitContext.HTTP_STATE_PROPERTY);
		if( httpState == null )
		{
		   httpState = new HttpState();
		   submitContext.setProperty( SubmitContext.HTTP_STATE_PROPERTY, httpState );
		   createdState = true;
		}
		
		HostConfiguration hostConfiguration = new HostConfiguration();
		
		submitContext.setProperty( POST_METHOD, postMethod );
		submitContext.setProperty( HTTP_CLIENT, httpClient );
		submitContext.setProperty( REQUEST_CONTENT, wsdlRequest.getRequestContent() );
		submitContext.setProperty( HOST_CONFIGURATION, hostConfiguration );
		submitContext.setProperty( WSDL_REQUEST, wsdlRequest );
		
		for( RequestFilter filter : filters )
		{
			filter.filterRequest( submitContext, wsdlRequest );
		}
		
		try
		{			
			Settings settings = wsdlRequest.getSettings();

			// get request content
			String requestContent = (String) submitContext.getProperty(REQUEST_CONTENT);

			// init
			requestContent = initRequest(wsdlRequest, postMethod, requestContent);
			
			// include request time?
			if (settings.getBoolean(HttpSettings.INCLUDE_REQUEST_IN_TIME_TAKEN))
				postMethod.initStartTime();
			
			//	custom http headers last so they can be overridden
			StringToStringMap headers = wsdlRequest.getRequestHeaders();
			for( String header : headers.keySet() )
			{
				String headerValue = headers.get( header );
				headerValue = PropertyExpansionRequestFilter.expandProperties( submitContext, headerValue );
				postMethod.setRequestHeader( header, headerValue );
			}
			
			//	do request
			httpClient.executeMethod(hostConfiguration, postMethod, httpState);
			
			Header responseContentTypeHeader = postMethod.getResponseHeader( "Content-Type" );
			
			if( !settings.getBoolean( WsdlRequest.INLINE_RESPONSE_ATTACHMENTS ) && 
				 responseContentTypeHeader != null && 
				 responseContentTypeHeader.getValue().toUpperCase().startsWith( "MULTIPART" ))
			{
				return new MimeMessageResponse( wsdlRequest, postMethod, requestContent );
			}
			else
			{
				return new SinglePartHttpResponse( wsdlRequest, postMethod, requestContent );
			}
		}
		finally
		{
			if (postMethod != null)
			{
				postMethod.releaseConnection();
			}
			else log.error( "PostMethod is null");
			
			if( createdState )
				submitContext.setProperty( SubmitContext.HTTP_STATE_PROPERTY, null );
		}		
	}

	private String initRequest(WsdlRequest wsdlRequest, TimeablePostMethod postMethod, String requestContent) throws Exception
	{
		MimeMultipart mp = null;
		
		StringToStringMap contentIds = new StringToStringMap();
		boolean isXOP = false;
		
		// preprocess only if neccessary
		if( wsdlRequest.isMtomEnabled() || wsdlRequest.getAttachmentCount() > 0 )
		{
			try
			{
				mp = new MimeMultipart();
				
				MessageXmlObject requestXmlObject = new MessageXmlObject(( WsdlOperation ) wsdlRequest.getOperation(), 
							requestContent, true);
				MessageXmlPart[] requestParts = requestXmlObject.getMessageParts();
				for (MessageXmlPart requestPart : requestParts)
				{
					if (AttachmentUtils.prepareMessagePart(wsdlRequest, mp, requestPart, contentIds))
						isXOP = true;
				}
				requestContent = requestXmlObject.getMessageContent();
			}
			catch (Exception e)
			{
				log.warn( "Failed to process inline/MTOM attachments; " + e );
			}			
		}
		
		// non-multipart request?
		if( !isXOP && (mp == null || mp.getCount() == 0 ) && wsdlRequest.getAttachmentCount() == 0 )
		{
			String encoding = StringUtils.unquote( wsdlRequest.getEncoding());
			byte[] content = encoding == null ? requestContent.getBytes() : requestContent.getBytes(encoding);
			postMethod.setRequestEntity(new ByteArrayRequestEntity(content));
		}
		else
		{
			// make sure..
			if( mp == null )
				mp = new MimeMultipart();
			
			// init root part
			initRootPart(wsdlRequest, requestContent, mp, isXOP);
			
			// init mimeparts
			AttachmentUtils.addMimeParts(wsdlRequest, mp, contentIds);
			
			// create request message
			MimeMessage message = new MimeMessage( AttachmentUtils.JAVAMAIL_SESSION );
			message.setContent( mp );
			message.saveChanges();
			MimeMessageRequestEntity mimeMessageRequestEntity = new MimeMessageRequestEntity( message, isXOP, wsdlRequest );
			postMethod.setRequestEntity( mimeMessageRequestEntity );
			postMethod.setRequestHeader( "Content-Type", mimeMessageRequestEntity.getContentType() );
			postMethod.setRequestHeader( "MIME-Version", "1.0" );
		}
		
		return requestContent;
	}

	/**
	 * Creates root BodyPart containing message
	 */
	
	private void initRootPart(WsdlRequest wsdlRequest, String requestContent, MimeMultipart mp, boolean isXOP) throws MessagingException
	{
		MimeBodyPart rootPart = new PreencodedMimeBodyPart( "8bit" );
		rootPart.setContentID( AttachmentUtils.ROOTPART_SOAPUI_ORG );
		mp.addBodyPart( rootPart, 0 );
		
		DataHandler dataHandler = new DataHandler( new WsdlRequestDataSource( wsdlRequest, requestContent, isXOP ) );
		rootPart.setDataHandler( dataHandler);
	}
	
	
}
