/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.submit.transports.http;

import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

import javax.activation.DataSource;
import javax.mail.BodyPart;
import javax.mail.MessagingException;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.MimeMultipart;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.model.iface.Attachment;
import com.eviware.soapui.settings.WsdlSettings;
import com.eviware.soapui.support.StringUtils;
import com.eviware.soapui.support.Tools;
import com.eviware.soapui.support.xml.XmlUtils;

public class MultipartMessageSupport 
{
	private final List<Attachment> attachments = new ArrayList<Attachment>();
	private Attachment rootPart;
	private MimeMessage message;
	private String responseContent;

	public MultipartMessageSupport( DataSource dataSource, String rootPartId ) throws MessagingException
	{
		MimeMultipart mp = new MimeMultipart( dataSource);
		message = new MimeMessage( AttachmentUtils.JAVAMAIL_SESSION );
		message.setContent( mp );
		
		for( int c = 0; c < mp.getCount(); c++ )
		{
			BodyPart bodyPart = mp.getBodyPart( c );
			
			if( bodyPart.getContentType().toUpperCase().startsWith( "MULTIPART/"))
			{
				MimeMultipart mp2 = new MimeMultipart( new BodyPartDataSource( bodyPart ));
				for( int i = 0; i < mp2.getCount(); i++ )
				{
					attachments.add( new BodyPartAttachment( mp2.getBodyPart(i)));
				}
			}
			else 
			{
				BodyPartAttachment attachment = new BodyPartAttachment( bodyPart);
				
				String[] contentIdHeaders = bodyPart.getHeader( "Content-ID");
				if( contentIdHeaders != null && contentIdHeaders.length > 0 && contentIdHeaders[0].equals( rootPartId ))
				{
					rootPart = attachment;
				}
				else
					attachments.add( attachment );
			}
		}
		
		// if no explicit root part has been set, use the first one in the result
		if( rootPart == null )
			rootPart = attachments.remove( 0 );
	}

	public Attachment[] getAttachments()
	{
		return attachments.toArray( new Attachment[attachments.size()]);
	}

	public Attachment getRootPart()
	{
		return rootPart;
	}

	public Attachment[] getAttachmentsForPart(String partName)
	{
		List<Attachment> results = new ArrayList<Attachment>();
		
		for( Attachment attachment : attachments )
		{
			if( attachment.getPart().equals( partName ))
				results.add( attachment );
		}
		
		return results.toArray( new Attachment[results.size()]);
	}

	public String getContentAsString()
	{
		if( rootPart == null )
			return null;
		
		if( responseContent == null )
		{
			try
			{
				InputStream in = rootPart.getInputStream();
				ByteArrayOutputStream data = Tools.readAll( in, Tools.READ_ALL );
				
				String contentType = rootPart.getContentType();
				if( contentType != null && contentType.indexOf( "charset=" ) > 0 )
				{
					try
					{
						int ix = contentType.indexOf( "charset=" );
						int ix2 = contentType.indexOf( ";", ix );
						
						String charset = ix2 == -1 ? contentType.substring( ix+8 ) : 
							contentType.substring( ix+8, ix2 );
						
						responseContent = data.toString( StringUtils.unquote( charset ) );
					}
					catch( Throwable e )
					{
						e.printStackTrace();
					}
				}
				
				if( responseContent == null )
				{
					responseContent = data.toString();
				}
				
				if( SoapUI.getSettings().getBoolean( WsdlSettings.PRETTY_PRINT_RESPONSE_MESSAGES ))
		   	{
					responseContent = XmlUtils.prettyPrintXml( responseContent );
		   	}
				
				return responseContent;
			}
			catch (Exception e)
			{
				e.printStackTrace();
			}
		}
		
		return responseContent;
	}
	
	public void setResponseContent(String responseContent)
	{
		this.responseContent = responseContent;
	}

	public Attachment getAttachmentWithContentId( String contentId )
	{
		for( Attachment attachment : attachments)
		   if( contentId.equals( attachment.getContentID() ))
		   	return attachment;
		
		return null;
	}
}