/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.teststeps;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;

import com.eviware.soapui.config.TestStepConfig;
import com.eviware.soapui.impl.wsdl.actions.support.ShowOnlineHelpAction;
import com.eviware.soapui.impl.wsdl.support.HelpUrls;
import com.eviware.soapui.impl.wsdl.testcase.WsdlTestCase;
import com.eviware.soapui.impl.wsdl.teststeps.actions.CloneTestStepAction;
import com.eviware.soapui.model.ModelItem;
import com.eviware.soapui.model.testsuite.TestRunContext;
import com.eviware.soapui.model.testsuite.TestRunner;
import com.eviware.soapui.model.testsuite.TestStepResult;
import com.eviware.soapui.model.testsuite.TestStepResult.TestStepStatus;
import com.eviware.soapui.support.UISupport;
import com.eviware.soapui.support.action.ActionSupport;
import com.eviware.soapui.support.xml.XmlObjectConfigurationBuilder;
import com.eviware.soapui.support.xml.XmlObjectConfigurationReader;

public class WsdlDelayTestStep extends WsdlTestStep
{
	private static final int DEFAULT_DELAY = 1000;
	private static final int DELAY_CHUNK = 100;
	private int delay = WsdlDelayTestStep.DEFAULT_DELAY;
	private int timeWaited = 0;
	private boolean canceled;

	public WsdlDelayTestStep(WsdlTestCase testCase, TestStepConfig config)
	{
		super(testCase, config, false);
		
		addAction( ActionSupport.SEPARATOR_ACTION );
		addAction( new SetWaitTimeAction(), true );
		addAction( new CloneTestStepAction( this, "DelayStep" ) );
		addAction( ActionSupport.SEPARATOR_ACTION );
      addAction( new ShowOnlineHelpAction( HelpUrls.DELAYSTEP_HELP_URL ));
		
		setIcon( UISupport.createImageIcon("/wait.gif"));

		if( config.getConfig() == null )
		{
			saveDelay( config );
		}
		else
		{
			readConfig( config );
		}
	}

	private void readConfig(TestStepConfig config)
	{
		XmlObjectConfigurationReader reader = new XmlObjectConfigurationReader( config.getConfig() );
		delay = reader.readInt( "delay", 1000 );
	}
	
	public String getName()
	{
		return super.getName() + " [" + (delay-timeWaited) + "ms]";
	}
	
	private void saveDelay(TestStepConfig config)
	{
		XmlObjectConfigurationBuilder builder = new XmlObjectConfigurationBuilder();
		builder.add( "delay", delay );
		config.setConfig( builder.finish() );
	}
	
	public void resetConfigOnMove(TestStepConfig config)
	{
		super.resetConfigOnMove( config );
		readConfig( config );
	}
	
	public int getDelay()
	{
		return delay;
	}
	
	public void setDelay( int delay )
	{
		if( this.delay == delay )
			return;
		
		String oldName = getName();
		
		this.delay = delay;
		saveDelay( getConfig() );
	   notifyPropertyChanged( ModelItem.NAME_PROPERTY, oldName, getName() );
	}

	public TestStepResult run(TestRunner testRunner, TestRunContext context)
	{
		WsdlTestStepResult result = new WsdlTestStepResult( this );
		result.startTimer();
		String oldName = getName(); 
		
		try
		{
			canceled = false;
			
			// sleep in chunks for canceling 
			for( timeWaited = 0; !canceled && timeWaited < delay; timeWaited += DELAY_CHUNK )
			{
				if( timeWaited % 1000 == 0 && context.getProperty( TestRunContext.LOAD_TEST_RUNNER ) == null )
				{
					notifyPropertyChanged( ModelItem.NAME_PROPERTY, oldName, getName() );
					oldName = getName();
				}
				
				if( timeWaited <= delay-DELAY_CHUNK )
					Thread.sleep( DELAY_CHUNK );
				else 
					Thread.sleep( delay % DELAY_CHUNK );
			}
		}
		catch (InterruptedException e)
		{
			e.printStackTrace();
		}
		
		result.stopTimer();
		result.setStatus( canceled ? TestStepStatus.CANCELED : TestStepStatus.OK );
		
		timeWaited = 0;
		
		if( context.getProperty( TestRunContext.LOAD_TEST_RUNNER ) == null )
			notifyPropertyChanged( ModelItem.NAME_PROPERTY, oldName, getName() );
		
		return result;
	}

	public boolean cancel()
	{
		canceled = true;
		return true;
	}
	
	private class SetWaitTimeAction extends AbstractAction
	{
		public SetWaitTimeAction()
		{
			super( "Set Delay Time" );
		}
		
		public void actionPerformed(ActionEvent e)
		{
			String value = UISupport.prompt( "Specify delay in milliseconds", "Set Delay", String.valueOf( delay ));
			if( value != null )
			{
				try
				{
					setDelay( Integer.parseInt(value) );
				}
				catch (NumberFormatException e1)
				{
					UISupport.showErrorMessage( e1 );
				}
			}
		}
	}
}
