 /*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.teststeps;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.ImageIcon;

import org.apache.xmlbeans.XmlObject;

import com.eviware.soapui.config.RequestAssertionConfig;
import com.eviware.soapui.impl.wsdl.panels.support.assertions.Assertable;
import com.eviware.soapui.impl.wsdl.panels.support.assertions.Assertable.AssertionStatus;
import com.eviware.soapui.impl.wsdl.submit.WsdlMessageExchange;
import com.eviware.soapui.impl.wsdl.teststeps.assertions.AssertionError;
import com.eviware.soapui.impl.wsdl.teststeps.assertions.AssertionException;
import com.eviware.soapui.impl.wsdl.teststeps.assertions.WsdlAssertionRegistry;
import com.eviware.soapui.model.iface.SubmitContext;
import com.eviware.soapui.model.settings.Settings;
import com.eviware.soapui.model.support.AbstractModelItem;
import com.eviware.soapui.support.UISupport;

/**
 * Base class for WsdlAssertions
 * 
 * @author Ole.Matzura
 */

public abstract class WsdlMessageAssertion extends AbstractModelItem
{
	private RequestAssertionConfig assertionConfig;
   private final Assertable assertable;
   private AssertionStatus assertionStatus = AssertionStatus.UNKNOWN;
   private com.eviware.soapui.impl.wsdl.teststeps.assertions.AssertionError [] assertionErrors;
   private ImageIcon validIcon;
   private ImageIcon failedIcon;
   private ImageIcon unknownIcon;
   
   public final static String STATUS_PROPERTY = WsdlMessageAssertion.class.getName() + "@status";  
   public final static String CONFIGURATION_PROPERTY = WsdlMessageAssertion.class.getName() + "@configuration";  
   
   public WsdlMessageAssertion(RequestAssertionConfig assertionConfig, Assertable modelItem)
   {
      this.assertionConfig = assertionConfig;
      this.assertable = modelItem;
      
      addAction( new RenameAssertionAction() );
      addAction( new RemoveAssertionAction() );
      if( isConfigurable() )
         addAction( new ConfigureAssertionAction() );
      
      validIcon = UISupport.createImageIcon("/valid_assertion.gif");
      failedIcon = UISupport.createImageIcon("/failed_assertion.gif");
      unknownIcon = UISupport.createImageIcon("/unknown_assertion.gif");      
   }
   
   protected XmlObject getConfiguration()
   {
      if( null == assertionConfig.getConfiguration())
      {
         assertionConfig.addNewConfiguration();
      }
      
      return assertionConfig.getConfiguration();
   }

   protected void setConfiguration( XmlObject configuration )
   {
	   XmlObject oldConfig = assertionConfig.getConfiguration();
      assertionConfig.setConfiguration( configuration );
      notifyPropertyChanged( WsdlMessageAssertion.CONFIGURATION_PROPERTY, oldConfig, configuration );
   }
   
   public String getName()
   {
      return assertionConfig.isSetName() ? assertionConfig.getName() : 
      	WsdlAssertionRegistry.getInstance().getAssertionNameForType( assertionConfig.getType());
   }

   public void setName(String name)
   {
      String old = getName();
      assertionConfig.setName( name );
      notifyPropertyChanged( NAME_PROPERTY, old, name );
   }
   
   public class RenameAssertionAction extends AbstractAction
   {
      public RenameAssertionAction()
      {
         super( "Rename" );
         putValue( Action.SHORT_DESCRIPTION, "Renames this assertion" );
      }
      
      public void actionPerformed(ActionEvent e)
		{
         String name = UISupport.prompt("Specify name for this assertion", "Rename Assertion", WsdlMessageAssertion.this.getName() );
         if( name == null || name.equals( WsdlMessageAssertion.this.getName() )) return;
         
         setName( name );
      }
   }
   
   public class RemoveAssertionAction extends AbstractAction
   {
      public RemoveAssertionAction()
      {
         super( "Remove" );
         putValue( Action.SHORT_DESCRIPTION, "Removes this assertion from its request" );
      }
      
      public void actionPerformed(ActionEvent e)
		{
         if( UISupport.confirm( "Remove assertion [" + WsdlMessageAssertion.this.getName() + "] from [" + 
               assertable.getTestStep().getName() + "]", "Remove Assertion" ))
         {
            assertable.removeAssertion( WsdlMessageAssertion.this );
         }
      }
   }
   
   public class ConfigureAssertionAction extends AbstractAction
   {
      public ConfigureAssertionAction()
      {
         super( "Configure" );
         putValue( Action.SHORT_DESCRIPTION, "Configures this assertion" );
      }
      
      public void actionPerformed(ActionEvent e)
		{
         configure();
      }
   }

   public AssertionStatus getStatus()
   {
      return assertionStatus;
   }

   public AssertionError[] getErrors()
   {
      return assertionErrors;
   }

   public AssertionStatus assertResponse( WsdlMessageExchange messageExchange, SubmitContext context)
   {
      AssertionStatus oldStatus = assertionStatus;
      ImageIcon oldIcon = getIcon();
      
      if( !messageExchange.hasResponse() )
      {
      	if( messageExchange.getOperation().isOneWay() )
      	{
      		assertionStatus = AssertionStatus.VALID;
	         assertionErrors = null;
      	}
      	else
      	{
      		assertionStatus = AssertionStatus.FAILED;
      		assertionErrors = new com.eviware.soapui.impl.wsdl.teststeps.assertions.AssertionError[] 
      	                    { new com.eviware.soapui.impl.wsdl.teststeps.assertions.AssertionError("null/empty response") };
      	}
      }
      else
      {
	      try
	      {
	      	internalAssertResponse( messageExchange, context );
	         assertionStatus = AssertionStatus.VALID;
	         assertionErrors = null;
	      }
	      catch ( AssertionException e )
	      {
	      	assertionStatus = AssertionStatus.FAILED;
	      	assertionErrors = e.getErrors();
	      }
	      catch (Throwable e)
	      {
	         assertionStatus = AssertionStatus.FAILED;
	         assertionErrors = new com.eviware.soapui.impl.wsdl.teststeps.assertions.AssertionError[] 
                           { new com.eviware.soapui.impl.wsdl.teststeps.assertions.AssertionError( e.getMessage() )};
	      }
      }
      
      notifyPropertyChanged( STATUS_PROPERTY, oldStatus, assertionStatus );
      notifyPropertyChanged( ICON_PROPERTY, oldIcon, getIcon() );
      
      return assertionStatus;
   }

   protected abstract String internalAssertResponse(WsdlMessageExchange messageExchange, SubmitContext context ) throws AssertionException;
   
   public AssertionStatus assertRequest( WsdlMessageExchange messageExchange, SubmitContext context)
   {
      AssertionStatus oldStatus = assertionStatus;
      ImageIcon oldIcon = getIcon();
      
      if( !messageExchange.hasRequest( true ) )
      {
      	assertionStatus = AssertionStatus.FAILED;
      	assertionErrors = new com.eviware.soapui.impl.wsdl.teststeps.assertions.AssertionError[] 
      	                    { new com.eviware.soapui.impl.wsdl.teststeps.assertions.AssertionError("null/empty response") };
      }
      else
      {
	      try
	      {
	      	internalAssertRequest( messageExchange, context );
	         assertionStatus = AssertionStatus.VALID;
	         assertionErrors = null;
	      }
	      catch ( AssertionException e )
	      {
	      	assertionStatus = AssertionStatus.FAILED;
	      	assertionErrors = e.getErrors();
	      }
	      catch (Throwable e)
	      {
	         assertionStatus = AssertionStatus.FAILED;
	         assertionErrors = new com.eviware.soapui.impl.wsdl.teststeps.assertions.AssertionError[] 
                           { new com.eviware.soapui.impl.wsdl.teststeps.assertions.AssertionError( e.getMessage() )};
	      }
      }
      
      notifyPropertyChanged( STATUS_PROPERTY, oldStatus, assertionStatus );
      notifyPropertyChanged( ICON_PROPERTY, oldIcon, getIcon() );
      
      return assertionStatus;
   }
   
   protected abstract String internalAssertRequest(WsdlMessageExchange messageExchange, SubmitContext context ) throws AssertionException;

   public boolean isConfigurable()
   {
      return false;
   }
   
   public boolean configure()
   {
   	return true;
   }

   public ImageIcon getIcon()
   {
      switch( getStatus() )
      {
         case FAILED : return failedIcon; 
         case UNKNOWN : return unknownIcon; 
         case VALID : return validIcon;
      }
      
      return null;
   }

	public void updateConfig(RequestAssertionConfig config)
	{
		this.assertionConfig = config;
	}

	public Settings getSettings()
	{
		return assertable.getTestStep().getSettings();
	}

	public void release()
	{
	}
	
	public Assertable getAssertable()
	{
		return assertable;
	}
}

  