/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.teststeps;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Iterator;
import java.util.List;

import javax.swing.ImageIcon;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.config.AttachmentConfig;
import com.eviware.soapui.config.CallConfig;
import com.eviware.soapui.config.RequestAssertionConfig;
import com.eviware.soapui.impl.settings.XmlBeansSettingsImpl;
import com.eviware.soapui.impl.wsdl.WsdlInterface;
import com.eviware.soapui.impl.wsdl.WsdlOperation;
import com.eviware.soapui.impl.wsdl.WsdlRequest;
import com.eviware.soapui.impl.wsdl.WsdlSubmitContext;
import com.eviware.soapui.impl.wsdl.actions.request.DeleteRequestAction;
import com.eviware.soapui.impl.wsdl.actions.request.RenameRequestAction;
import com.eviware.soapui.impl.wsdl.panels.support.assertions.Assertable;
import com.eviware.soapui.impl.wsdl.panels.support.assertions.AssertionsListener;
import com.eviware.soapui.impl.wsdl.submit.transports.http.WsdlResponse;
import com.eviware.soapui.impl.wsdl.testcase.WsdlTestCase;
import com.eviware.soapui.impl.wsdl.teststeps.actions.AddAssertionAction;
import com.eviware.soapui.impl.wsdl.teststeps.assertions.WsdlAssertionRegistry;
import com.eviware.soapui.impl.wsdl.teststeps.assertions.WsdlAssertionRegistry.AssertionType;
import com.eviware.soapui.model.iface.Submit;
import com.eviware.soapui.model.iface.SubmitContext;
import com.eviware.soapui.monitor.TestMonitor;
import com.eviware.soapui.support.UISupport;

/**
 * WsdlRequest extension that adds WsdlAssertions
 * 
 * @author Ole.Matzura
 */

public class WsdlTestRequest extends WsdlRequest implements PropertyChangeListener, Assertable
{
   public static final String RESPONSE_PROPERTY = WsdlTestRequest.class.getName() + "@response";
   public static final String STATUS_PROPERTY = WsdlTestRequest.class.getName() + "@status";
   
   private ImageIcon validRequestIcon;
   private ImageIcon failedRequestIcon;
   private ImageIcon disabledRequestIcon;
   private ImageIcon unknownRequestIcon;
   private AssertionStatus currentStatus;
   private final WsdlTestRequestStep testStep;
   
   private AssertionsSupport assertionsSupport;
	private WsdlResponseMessageExchange messageExchange;
   
   public WsdlTestRequest( WsdlOperation operation, CallConfig callConfig, WsdlTestRequestStep testStep )
   {
      super( operation, callConfig );
      
      setSettings( new XmlBeansSettingsImpl( this, testStep.getSettings(), callConfig.getSettings() ));
      
      this.testStep = testStep;
      
      initAssertions();
      initIcons();
   }
   
   public WsdlTestCase getTestCase()
   {
      return testStep.getTestCase();
   }

   protected void initIcons()
   {
      validRequestIcon = UISupport.createImageIcon("/valid_request.gif");
      failedRequestIcon = UISupport.createImageIcon("/invalid_request.gif");
      unknownRequestIcon = UISupport.createImageIcon("/unknown_request.gif");
      disabledRequestIcon = UISupport.createImageIcon("/disabled_request.gif");
   }
   
   protected RequestIconAnimator initIconAnimator()
   {
   	return new TestRequestIconAnimator();
   }
   
   protected void initActions()
   {
      addAction( new RenameRequestAction( this ));
      addAction( new DeleteRequestAction( this ));
      addAction( new AddAssertionAction( this ));
   }

   private void initAssertions()
   {
   	assertionsSupport = new AssertionsSupport( this, getConfig().getAssertionList() );
   }
   
   public int getAssertionCount()
   {
      return assertionsSupport.getAssertionCount();
   }

   public WsdlMessageAssertion getAssertionAt(int c)
   {
      return assertionsSupport.getAssertionAt( c );
   }

   public void setResponse(WsdlResponse response, SubmitContext context)
   {
      super.setResponse( response, context );
      
      assertResponse( context );
   }

   public void assertResponse( SubmitContext context ) 
   {
   	PropertyChangeNotifier notifier = new PropertyChangeNotifier();
   	messageExchange = new WsdlResponseMessageExchange( this );
   	
		// assert!
      for (Iterator<WsdlMessageAssertion> iter = assertionsSupport.iterator(); iter.hasNext();)
      {
         iter.next().assertResponse( messageExchange, context );
      }

      notifier.notifyChange();
   }

	private class PropertyChangeNotifier
   {
	   private AssertionStatus oldStatus;
	   private ImageIcon oldIcon;

	   public PropertyChangeNotifier()
	   {
		   oldStatus = getAssertionStatus();
		   oldIcon = getIcon();
	   }
	   
	   public void notifyChange()
	   {
		   AssertionStatus newStatus = getAssertionStatus();
	      ImageIcon newIcon = getIcon();
	      
	      if( oldStatus != newStatus )
	         notifyPropertyChanged( STATUS_PROPERTY, oldStatus, newStatus );
	      
	      if( oldIcon != newIcon )
	         notifyPropertyChanged( ICON_PROPERTY, oldIcon, getIcon() );
	   }
   }
   
   public WsdlMessageAssertion addAssertion(String assertionLabel)
   {
      PropertyChangeNotifier notifier = new PropertyChangeNotifier();
    
      try
      {
      	RequestAssertionConfig assertionConfig = (RequestAssertionConfig) getConfig().addNewAssertion();
         assertionConfig.setType( WsdlAssertionRegistry.getInstance().getAssertionTypeForName( assertionLabel ));
         
      	WsdlMessageAssertion assertion = assertionsSupport.addWsdlAssertion( assertionConfig );
      	assertionsSupport.fireAssertionAdded( assertion );
      	
         if( getResponse() != null )
         {
            assertion.assertResponse( new WsdlResponseMessageExchange( this ), new WsdlSubmitContext( testStep ) );
            notifier.notifyChange();
         }
         
         return assertion;
      }
      catch (Exception e)
      {
         e.printStackTrace();
         return null;
      }
   }
   
	public void removeAssertion(WsdlMessageAssertion assertion)
   {
      PropertyChangeNotifier notifier = new PropertyChangeNotifier();
      
      try
      {
      	int ix = assertionsSupport.removeAssertion( assertion );
      	getConfig().removeAssertion( ix );
      }
      finally
      {
	      assertion.release();
	      notifier.notifyChange();
      }
   }

   public AssertionStatus getAssertionStatus()
   {
      currentStatus = AssertionStatus.UNKNOWN;
      
      if( messageExchange != null )
      {
      	if( !messageExchange.hasResponse() && !getOperation().isOneWay() )
   		{
   			currentStatus = AssertionStatus.FAILED;
   		}
      }
      else  return currentStatus;
      
      int cnt = getAssertionCount();
      if( cnt == 0 ) 
      	return currentStatus;
      
      for( int c = 0; c < cnt; c++ )
      {
         if( getAssertionAt( c ).getStatus() == AssertionStatus.FAILED )
         {
            currentStatus = AssertionStatus.FAILED;
            break;
         }
      }
      
      if( currentStatus == AssertionStatus.UNKNOWN )
      	currentStatus = AssertionStatus.VALID;
      
      return currentStatus;
   }

   public ImageIcon getIcon()
   {
   	TestMonitor testMonitor = SoapUI.getTestMonitor();
		if( testMonitor != null && testMonitor.hasRunningLoadTest( testStep.getTestCase()) )
   		return disabledRequestIcon;
   	
   	ImageIcon icon = getIconAnimator().getIcon();
      if( icon == getIconAnimator().getBaseIcon() )
      {
         AssertionStatus status = getAssertionStatus();
         if( status == AssertionStatus.VALID ) return validRequestIcon; 
         else if( status == AssertionStatus.FAILED ) return failedRequestIcon; 
         else if( status == AssertionStatus.UNKNOWN ) return unknownRequestIcon; 
      }
      
      return icon;
   }

   public void propertyChange(PropertyChangeEvent evt)
   {
	  if( evt.getPropertyName().equals( WsdlMessageAssertion.CONFIGURATION_PROPERTY ))
		  assertResponse( new WsdlSubmitContext( testStep ));
   }

   public void addAssertionsListener( AssertionsListener listener )
   {
   	assertionsSupport.addAssertionsListener( listener );
   }
   
   public void removeAssertionsListener( AssertionsListener listener )
   {
   	assertionsSupport.removeAssertionsListener( listener );
   }
   
   /**
    * Called when a testrequest is moved in a testcase
    */
   
	public void updateConfig(CallConfig request)
	{
		super.updateConfig(request);
		
		assertionsSupport.updateConfig( getConfig().getAssertionList() );
      
      List<AttachmentConfig> attachmentConfigs = getConfig().getAttachmentList();
      for( int i = 0; i < attachmentConfigs.size(); i++ )
      {
      	AttachmentConfig config = attachmentConfigs.get( i );
      	attachments.get( i ).updateConfig( config );
      }
	}

	public void release()
	{
		super.release();
		assertionsSupport.release();
	}

	public String getAssertableContent()
	{
		return getResponse() == null ? null : getResponse().getContentAsString();
	}

	public WsdlTestRequestStep getTestStep()
	{
		return testStep;
	}

	public WsdlInterface getInterface()
	{
		return getOperation().getInterface();
	}
	
	protected class TestRequestIconAnimator extends RequestIconAnimator 
	{
		public boolean beforeSubmit(Submit submit, SubmitContext context) 
		{
			if( SoapUI.getTestMonitor() != null && SoapUI.getTestMonitor().hasRunningLoadTest( getTestCase() ))
				return true;
			
			return super.beforeSubmit( submit, context );
		}

		public void afterSubmit(Submit submit, SubmitContext context) 
		{
			if( submit.getRequest() == getTarget() )
				stop();
		}
	}

	public AssertionType getAssertionType()
	{
		return AssertionType.RESPONSE;
	}
}
